# -*- coding: utf-8 -*-

import sys

from ScoutSuite.providers.aws.provider import AWSProvider
from ScoutSuite.providers.gcp.provider import GCPProvider
from ScoutSuite.providers.azure.provider import AzureProvider

providers_dict = {'aws': 'AWSProvider',
                  'gcp': 'GCPProvider',
                  'azure': 'AzureProvider'}

def get_provider(provider,
                 profile=None,
                 project_id=None, folder_id=None, organization_id=None,
                 report_dir=None, timestamp=None, services=None, skipped_services=None, thread_config=4, **kwargs):
    """
    Returns an instance of the requested provider.

    :param provider: a string indicating the provider
    :return: a child instance of the BaseProvider class or None if no object implemented
    """
    services = [] if services is None else services
    skipped_services = [] if skipped_services is None else skipped_services

    provider_class = providers_dict.get(provider)
    provider_object = getattr(sys.modules[__name__], provider_class)
    provider_instance = provider_object(profile=profile,
                                        project_id=project_id,
                                        folder_id=folder_id,
                                        organization_id=organization_id,
                                        report_dir=report_dir,
                                        timestamp=timestamp,
                                        services=services,
                                        skipped_services=skipped_services,
                                        thread_config=thread_config,
                                        **kwargs)

    return provider_instance



# -*- coding: utf-8 -*-

import json
import os
import re
import time

from opinel.utils.aws import connect_service, handle_truncated_response
from opinel.utils.console import printDebug, printInfo, printError, printException, prompt_4_yes_no
from opinel.utils.fs import read_file
from opinel.utils.globals import snake_to_camel, snake_to_words

re_iam_capability = re.compile('.*?AWS::IAM.*?', re.DOTALL | re.MULTILINE)

def create_cloudformation_resource_from_template(api_client, resource_type, name, template_path, template_parameters=[], tags=[], quiet=False, wait_for_completion = False, need_on_failure=False):
    """

    :param callback:
    :param name:
    :param template_path:
    :param template_parameters:
    :param quiet:
    :return:
    """
    create = getattr(api_client, 'create_%s' % resource_type)
    api_resource_type = snake_to_camel(resource_type)
    # Add a timestamps
    tags.append({'Key': 'OpinelTimestamp', 'Value': str(time.time())})
    params = prepare_cloudformation_params(name, template_path, template_parameters, api_resource_type, tags)
    if not quiet:
        printInfo('Creating the %s %s..' % (resource_type, name))
    response = create(**params)
    resource_id_attribute = '%sId' % api_resource_type
    resource_id = response[resource_id_attribute] if resource_id_attribute in response else None
    operation_id = response['OperationId'] if 'OperationId' in response else None
    if wait_for_completion:
        cloudformation_wait(api_client, resource_type, name, operation_id)
    return resource_id


def create_stack(api_client, stack_name, template_path, template_parameters=[], tags=[], quiet=False, wait_for_completion = False):
    """

    :param api_client:
    :param stack_name:
    :param template_path:
    :param template_parameters:         List of parameter keys and values
    :param quiet:
    :return:
    """
    return create_cloudformation_resource_from_template(api_client, 'stack', stack_name, template_path, template_parameters, tags, quiet, wait_for_completion, need_on_failure=True)


def create_or_update_stack(api_client, stack_name, template_path, template_parameters=[], tags=[], quiet=False, wait_for_completion = False):
    """

    :param api_client:
    :param stack_name:
    :param template_path:
    :param template_parameters:         List of parameter keys and values
    :param quiet:
    :return:
    """
    try:
        stack = api_client.describe_stacks(StackName = stack_name)
        printInfo('Stack already exists... ', newLine = False)
        stack_id = update_stack(api_client, stack_name, template_path, template_parameters, quiet, wait_for_completion)
    except Exception as e:
        if hasattr(e, 'response') and type(e.response) == dict and 'Error' in            e.response and e.response['Error']['Code'] == 'ValidationError':
            stack_id = create_stack(api_client, stack_name, template_path, template_parameters, tags, quiet, wait_for_completion)
        else:
            stack_id = None
            printException(e)
    return stack_id



def create_stack_set(api_client, stack_set_name, template_path, template_parameters=[], tags=[], quiet=False, wait_for_completion = False):
    """

    :param api_client:
    :param stack_set_name:
    :param template_path:
    :param template_parameters:
    :param quiet:
    :return:
    """
    return create_cloudformation_resource_from_template(api_client, 'stack_set', stack_set_name, template_path, template_parameters, tags, quiet, wait_for_completion)


def create_or_update_stack_set(api_client, stack_set_name, template_path, template_parameters=[], tags=[], quiet=False, wait_for_completion = False):
    """

    :param api_client:
    :param stack_name:
    :param template_path:
    :param template_parameters:         List of parameter keys and values
    :param quiet:
    :return:
    """
    operation_id = stack_set_id = None
    try:
        stack_set = api_client.describe_stack_set(StackSetName = stack_set_name)
        printInfo('Stack set already exists... ', newLine = False)
        operation_id = update_stack_set(api_client, stack_set_name, template_path, template_parameters, quiet, wait_for_completion)
    except Exception as e:
        if hasattr(e, 'response') and type(e.response) == dict and 'Error' in e.response and e.response['Error']['Code'] == 'StackSetNotFoundException':
            stack_set_id = create_stack_set(api_client, stack_set_name, template_path, template_parameters, tags, quiet, wait_for_completion)
        else:
            printException(e)
    return (stack_set_id, operation_id)


def create_stack_instances(api_client, stack_set_name, account_ids, regions, quiet=False):
    """

    :param api_client:
    :param stack_set_name:
    :param account_ids:
    :param regions:
    :return:
    """
    operation_preferences = {'FailureTolerancePercentage': 100,
       'MaxConcurrentPercentage': 100
       }
    if not quiet:
        printInfo('Creating stack instances in %d regions and %d accounts...' % (len(regions), len(account_ids)))
        printDebug(' %s' % ', '.join(regions))
    response = api_client.create_stack_instances(StackSetName=stack_set_name, Accounts=account_ids, Regions=regions, OperationPreferences=operation_preferences)
    if not quiet:
        printInfo('Successfully started operation Id %s' % response['OperationId'])
    return response['OperationId']


def delete_stack_set(api_client, stack_set_name, timeout = 60 * 5):
    """
    """
    printDebug('Deleting stack set %s' % stack_set_name)
    # Check for instances
    stack_instances = handle_truncated_response(api_client.list_stack_instances, {'StackSetName': stack_set_name}, ['Summaries'])['Summaries']
    account_ids = []
    regions = []
    if len(stack_instances) > 0:
        for si in stack_instances:
            if si['Account'] not in account_ids:
                account_ids.append(si['Account'])
            if si['Region'] not in regions:
                regions.append(si['Region'])
        operation_id = api_client.delete_stack_instances(StackSetName = stack_set_name, Accounts = account_ids, Regions = regions, RetainStacks = False)['OperationId']
        wait_for_operation(api_client, stack_set_name, operation_id)
    api_client.delete_stack_set(StackSetName = stack_set_name)


def get_stackset_ready_accounts(credentials, account_ids, quiet=True):
    """
    Verify which AWS accounts have been configured for CloudFormation stack set by attempting to assume the stack set execution role

    :param credentials:                 AWS credentials to use when calling sts:assumerole
    :param org_account_ids:             List of AWS accounts to check for Stackset configuration

    :return:                            List of account IDs in which assuming the stackset execution role worked
    """
    api_client = connect_service('sts', credentials, silent=True)
    configured_account_ids = []
    for account_id in account_ids:
        try:
            role_arn = 'arn:aws:iam::%s:role/AWSCloudFormationStackSetExecutionRole' % account_id
            api_client.assume_role(RoleArn=role_arn, RoleSessionName='opinel-get_stackset_ready_accounts')
            configured_account_ids.append(account_id)
        except Exception as e:
            pass

    if len(configured_account_ids) != len(account_ids) and not quiet:
        printInfo('Only %d of these accounts have the necessary stack set execution role:' % len(configured_account_ids))
        printDebug(str(configured_account_ids))
    return configured_account_ids


def make_awsrecipes_stack_name(template_path):
    """

    :param template_path:
    :return:
    """
    return make_prefixed_stack_name('AWSRecipes', template_path)


def make_opinel_stack_name(template_path):
    """

    :param template_path:"
    :return:
    """
    return make_prefixed_stack_name('Opinel', template_path)


def make_prefixed_stack_name(prefix, template_path):
    """

    :param prefix:
    :param template_path:
    """
    parts = os.path.basename(template_path).split('-')
    parts = parts if len(parts) == 1 else parts[:-1]
    return ('%s-%s' % (prefix, '-'.join(parts))).split('.')[0]


def prepare_cloudformation_params(stack_name, template_path, template_parameters, resource_type, tags=[], need_on_failure=False):
    """

    :param api_client:
    :param stack_name:
    :param template_path:
    :param template_parameters:         List of parameter keys and values
    :param quiet:
    :return:
    """
    printDebug('Reading CloudFormation template from %s' % template_path)
    template_body = read_file(template_path)
    params = {}
    params['%sName' % resource_type] = stack_name
    params['TemplateBody'] = template_body
    if len(template_parameters):
        params['Parameters'] = []
        it = iter(template_parameters)
        for param in it:
            printError('Param:: %s' % param)
            params['Parameters'].append({'ParameterKey': param,'ParameterValue': next(it)})

    if len(tags):
        params['Tags'] = tags
    if re_iam_capability.match(template_body):
        params['Capabilities'] = [
         'CAPABILITY_NAMED_IAM']
    if need_on_failure:
        params['OnFailure'] = 'ROLLBACK'
    return params


def update_stack(api_client, stack_name, template_path, template_parameters = [], quiet = False, wait_for_completion = False):
    """

    :param api_client:
    :param stack_name:
    :param template_path:
    :param template_parameters:         List of parameter keys and values
    :param quiet:
    :return:
    """
    update_cloudformation_resource_from_template(api_client, 'stack', stack_name, template_path, template_parameters, quiet = quiet, wait_for_completion = wait_for_completion)


def update_stack_set(api_client, stack_set_name, template_path, template_parameters=[], quiet=False, wait_for_completion = False):
    """

    :param api_client:
    :param stack_set_name:
    :param template_path:
    :param template_parameters:
    :param quiet:
    :return:
    """
    return update_cloudformation_resource_from_template(api_client, 'stack_set', stack_set_name, template_path, template_parameters, [], quiet, wait_for_completion)


def update_cloudformation_resource_from_template(api_client, resource_type, name, template_path, template_parameters=[], tags=[], quiet=False, wait_for_completion = False):
    """

    :param callback:
    :param name:
    :param template_path:
    :param template_parameters:
    :param quiet:
    :return:
    """
    try:
        update = getattr(api_client, 'update_%s' % resource_type)
        api_resource_type = snake_to_camel(resource_type)
        # Add a timestamps
        tags.append({'Key': 'OpinelTimestamp', 'Value': str(time.time())})
        params = prepare_cloudformation_params(name, template_path, template_parameters, api_resource_type, tags)
        if not quiet:
            printInfo('Updating the %s...' % resource_type, newLine=False)
        response = update(**params)
        operation_id = response['OperationId'] if resource_type == 'stack_set' else None
        if wait_for_completion:
            cloudformation_wait(api_client, resource_type, name, operation_id)

    except Exception as e:
        if api_resource_type == 'Stack' and hasattr(e, 'response') and type(e.response == dict) and e.response['Error']['Code'] == 'ValidationError' and e.response['Error']['Message'] == 'No updates are to be performed.':
            printInfo(' Already up to date.')
        else:
            printException(e)
            printError(' Failed.')


def wait_for_operation(api_client, stack_set_name, operation_id, timeout = 5 * 60, increment = 5):
    printDebug('Waiting for operation %s on stack set %s...' % (operation_id, stack_set_name))
    timer = 0
    status = ''
    while True:
        if timer >= timeout:
            printError('Timed out.')
            break
        info = api_client.describe_stack_set_operation(StackSetName = stack_set_name, OperationId = operation_id)
        status = info['StackSetOperation']['Status']
        if status not in ['RUNNING', 'STOPPING']:
            break
        printError('Operation status is \'%s\'... waiting %d seconds until next check...' % (status, increment))
        time.sleep(increment)
        timer += increment
    return 'Operation %s is %s' % (operation_id, status)


def wait_for_stack_set(api_client, stack_set_name, timeout = 60, increment = 5):
    printDebug('Waiting for stack set %s to be ready...' % stack_set_name)
    timer = 0
    while True:
        if timer >= timeout:
            printError('Timed out.')
            break
        printError('Checking the stack set\'s status...')
        time.sleep(increment)
        timer += increment
        info = api_client.describe_stack_set(StackSetName = stack_set_name)
        if info['StackSet']['Status'] == 'ACTIVE':
            break


def still_running(callback, params, resource_type):
    rc = True
    response = callback(**params)
    if resource_type == 'stack':
        status = response['Stacks'][0]['StackStatus']
        if status.endswith('_COMPLETE') or status.endswith('_FAILED'):
            rc = False
    elif resource_type == 'stack_set':
        status = response['StackSet']['Status']
        if status == 'ACTIVE':
            rc = False
    elif resource_type == 'operation':
        status = response['StackSetOperation']['Status']
        if status != 'RUNNING':
            rc = False
    return (rc, status)


def cloudformation_wait(api_client, resource_type, resource_name, operation_id = None, timeout = 5 * 60, increment = 5):
    if resource_type == 'stack':
        callback = api_client.describe_stacks
        params = {'StackName': resource_name}
    elif resource_type == 'stack_set':
        params = {'StackSetName': resource_name}
        if operation_id:
            callback = api_client.describe_stack_set_operation
            params['OperationId'] = operation_id
            resource_type = 'operation'
        else:
            callback = api_client.describe_stack_set
    else:
        printError('Unknown resource type: %s' % resource_type)
        return
    timer = 0
    while True:
        if timer >= timeout:
            printError('Timed out.')
            break
        rc, status = still_running(callback, params, resource_type)
        if rc == False:
            printInfo('Status: %s' % status)
            break
        printInfo('Status: %s... waiting %d seconds until next check...' % (status, increment))
        timer += increment
        time.sleep(increment)



from ScoutSuite.providers.aws.configs.regions import RegionalServiceConfig, RegionConfig, api_clients


class KMSRegionConfig(RegionConfig):
    """
    KMS Configuration for a single AWS region
    """

    def parse_key(self, global_params, region, key):
        """
        Parse a single Key Management Service

        :param global_params:           Parameters shared for all regions
        :param region:                  Name of the AWS region
        :param key:                     Key
        """

        customer_master_key = {'id': key.pop('KeyId'), 'arn': key.pop('KeyArn')}

        api_client = api_clients[region]

        rotation_status = api_client.get_key_rotation_status(KeyId=customer_master_key['id'])
        customer_master_key['rotation_enabled'] = rotation_status['KeyRotationEnabled']

        key_information = api_client.describe_key(KeyId=customer_master_key['id'])
        customer_master_key['description'] = key_information['KeyMetadata']['Description']

        alias = api_client.list_aliases(KeyId=customer_master_key['id'])
        if len(alias['Aliases']):
            customer_master_key['name'] = alias['Aliases'][0]['AliasName'][6:len(alias['Aliases'][0]['AliasName'])]
        else:
            customer_master_key['name'] = 'unnamed key'

        self.keys[len(self.keys)] = customer_master_key


########################################
# KMSConfig
########################################

class KMSConfig(RegionalServiceConfig):
    """
    KMS configuration for all AWS regions
    """

    region_config_class = KMSRegionConfig

    def __init__(self, service_metadata, thread_config=4):
        super(KMSConfig, self).__init__(service_metadata, thread_config)



# -*- coding: utf-8 -*-

import os

condition_operators = [ 'and', 'or' ]

awsscout2_rules_data_dir  = os.path.join(os.path.dirname(__file__), 'data')
awsscout2_conditions_dir  = os.path.join(awsscout2_rules_data_dir, 'conditions')
awsscout2_filters_dir     = os.path.join(awsscout2_rules_data_dir, 'filters')
awsscout2_findings_dir    = os.path.join(awsscout2_rules_data_dir, 'findings')
awsscout2_rulesets_dir    = os.path.join(awsscout2_rules_data_dir, 'rulesets')



# -*- coding: utf-8 -*-

from opinel.utils.aws import handle_truncated_response

from ScoutSuite.providers.aws.configs.base import AWSBaseConfig



class Route53DomainsConfig(AWSBaseConfig):
    """
    Object that holds the Route53Domains configuration
    """

    targets = (
        ('domains', 'Domains', 'list_domains', {}, False),
    )

    def __init__(self, target_config):
        self.domains = {}
        self.domains_count = 0
        super(Route53DomainsConfig, self).__init__(target_config)



    ########################################
    ##### Domains
    ########################################
    def parse_domains(self, domain, params):
        """
        Parse a single Route53Domains domain
        """
        domain_id = self.get_non_provider_id(domain['DomainName'])
        domain['name'] = domain.pop('DomainName')
        #TODO: Get Dnssec info when available
        #api_client = params['api_client']
        #details = api_client.get_domain_detail(DomainName = domain['name'])
        #get_keys(details, domain, ['Dnssec'])
        self.domains[domain_id] = domain



class Route53Config(AWSBaseConfig):
    """
    Object that holds the Route53 configuration
    """

    targets = (
        ('hosted_zones', 'HostedZones', 'list_hosted_zones', {}, False),
    )

    def __init__(self, target_config):
        self.hosted_zones = {}
        self.hosted_zones_count = 0
        super(Route53Config, self).__init__(target_config)



    ########################################
    ##### hosted_zoness
    ########################################
    def parse_hosted_zones(self, hosted_zone, params):
        """
        Parse a single Route53hosted_zoness hosted_zones
        """
        # When resuming upon throttling error, skip if already fetched
        hosted_zone_id = hosted_zone.pop('Id')
        hosted_zone['name'] = hosted_zone.pop('Name')
        api_client = params['api_client']
        record_sets = handle_truncated_response(api_client.list_resource_record_sets, {'HostedZoneId': hosted_zone_id}, ['ResourceRecordSets'])
        hosted_zone.update(record_sets)
        #print(str(record_sets))
        #record_sets = api_client.list_resource_record_sets()
        #hosted_zone['RecordSets'] = record_sets['Resourc']
        self.hosted_zones[hosted_zone_id] = hosted_zone



# -*- coding: utf-8 -*-

from ScoutSuite.providers.aws.configs.regions import RegionalServiceConfig, RegionConfig, api_clients



########################################
# DirectConnectRegionConfig
########################################

class DirectConnectRegionConfig(RegionConfig):
    """
    DirectConnect configuration for a single AWS region
    """

    def parse_connection(self, global_params, region, connection):
        """
        Parse a single connection and fetch additional attributes

        :param global_params:           Parameters shared for all regions
        :param region:                  Name of the AWS region
        :param connection_url:               URL of the AWS connection
        """
        connection['id'] = connection.pop('connectionId')
        connection['name'] = connection.pop('connectionName')
        self.connections[connection['id']] = connection



########################################
# DirectConnectConfig
########################################

class DirectConnectConfig(RegionalServiceConfig):
    """
    DirectConnect configuration for all AWS regions
    """

    region_config_class = DirectConnectRegionConfig

    def __init__(self, service_metadata, thread_config = 4):
        super(DirectConnectConfig, self).__init__(service_metadata, thread_config)



# -*- coding: utf-8 -*-
"""
Single-service rule processing functions
"""

from six import string_types
import copy
import re

from opinel.utils.conditions import pass_condition
from opinel.utils.console import printError, printException

from ScoutSuite.core import condition_operators
from ScoutSuite.providers.base.configs.browser import get_value_at

re_get_value_at = re.compile(r'_GET_VALUE_AT_\((.*?)\)')
re_nested_get_value_at = re.compile(r'_GET_VALUE_AT_\(.*')


def fix_path_string(all_info, current_path, path_to_value):
    # handle nested _GET_VALUE_AT_...
    while True:
        dynamic_path = re_get_value_at.findall(path_to_value)
        if len(dynamic_path) == 0:
            break
        for dp in dynamic_path:
            tmp = dp
            while True:
                nested = re_nested_get_value_at.findall(tmp)
                if len(nested) == 0:
                    break
                tmp = nested[0].replace('_GET_VALUE_AT_(', '', 1)
            dv = get_value_at(all_info, current_path, tmp)
            path_to_value = path_to_value.replace('_GET_VALUE_AT_(%s)' % tmp, dv)
    return path_to_value


def recurse(all_info, current_info, target_path, current_path, config, add_suffix=False):
    """
    Recursively test conditions for a path.
    In order to do this, needs to evaluate all the `id` possibilities.

    When the value in the path is `id`, this represents either a key for a dict or an index for a list.

    When the is `id`:
    - For a dict return value at key
    - For a list, return the list
    When the value ends in `id.`:
    - For a dict, return a list of keys
    - For a list, return value at the index indicated by id
    `
    :param all_info:        All of the services' data
    :param current_info:    ?
    :param target_path:     The path that is being tested
    :param current_path:
    :param config:          The Rule object that is being tested
    :param add_suffix:      ?
    :return:
    """
    results = []
    if len(target_path) == 0:
        # Dashboard: count the number of processed resources here
        setattr(config, 'checked_items', getattr(config, 'checked_items') + 1)
        # Test for conditions...
        if pass_conditions(all_info, current_path, copy.deepcopy(config.conditions)):
            if add_suffix and hasattr(config, 'id_suffix'):
                suffix = fix_path_string(all_info, current_path, config.id_suffix)
                current_path.append(suffix)
            results.append('.'.join(current_path))
        # Return the flagged items...
        return results
    target_path = copy.deepcopy(target_path)
    dbg_target_path = copy.deepcopy(target_path)
    current_path = copy.deepcopy(current_path)
    attribute = target_path.pop(0)
    if type(current_info) == dict:
        if attribute in current_info:
            split_path = copy.deepcopy(current_path)
            split_path.append(attribute)
            results = results + recurse(all_info, current_info[attribute], target_path, split_path, config, add_suffix)
        elif attribute == 'id':
            for key in current_info:
                split_target_path = copy.deepcopy(target_path)
                split_current_path = copy.deepcopy(current_path)
                split_current_path.append(key)
                split_current_info = current_info[key]
                results = results + recurse(all_info, split_current_info, split_target_path, split_current_path,
                                            config, add_suffix)
    # To handle lists properly, I would have to make sure the list is properly ordered and I can use the index to
    # consistently access an object... Investigate (or do not use lists)
    elif type(current_info) == list:
        for index, split_current_info in enumerate(current_info):
            split_current_path = copy.deepcopy(current_path)
            split_current_path.append(str(index))
            results = results + recurse(all_info, split_current_info, copy.deepcopy(target_path), split_current_path,
                                        config, add_suffix)
    # Python 2-3 compatible way to check for string type
    elif isinstance(current_info, string_types):
        split_current_path = copy.deepcopy(current_path)
        results = results + recurse(all_info, current_info, [], split_current_path,
                                    config, add_suffix)
    else:
        printError('Error: unhandled case, typeof(current_info) = %s' % type(current_info))
        printError('Path: %s' % current_path)
        printError('Object: %s' % str(current_info))
        printError('Entry target path: %s' % str(dbg_target_path))
        raise Exception
    return results


def pass_conditions(all_info, current_path, conditions, unknown_as_pass_condition=False):
    """
    Check that all conditions are passed for the current path.

    :param all_info:        All of the services' data
    :param current_path:    The value of the `path` variable defined in the finding file
    :param conditions:      The conditions to check as defined in the finding file
    :param unknown_as_pass_condition:   Consider an undetermined condition as passed
    :return:
    """
    result = False
    if len(conditions) == 0:
        return True
    condition_operator = conditions.pop(0)
    for condition in conditions:
        if condition[0] in condition_operators:
            res = pass_conditions(all_info, current_path, condition, unknown_as_pass_condition)
        else:
            # Conditions are formed as "path to value", "type of test", "value(s) for test"
            path_to_value, test_name, test_values = condition
            path_to_value = fix_path_string(all_info, current_path, path_to_value)
            target_obj = get_value_at(all_info, current_path, path_to_value)
            if type(test_values) != list:
                dynamic_value = re_get_value_at.match(test_values)
                if dynamic_value:
                    test_values = get_value_at(all_info, current_path, dynamic_value.groups()[0], True)
            try:
                res = pass_condition(target_obj, test_name, test_values)
            except Exception as e:
                res = True if unknown_as_pass_condition else False
                printError('Unable to process testcase \'%s\' on value \'%s\', interpreted as %s.' % (
                    test_name, str(target_obj), res))
                printException(e, True)
        # Quick exit and + false
        if condition_operator == 'and' and not res:
            return False
        # Quick exit or + true
        if condition_operator == 'or' and res:
            return True
    # Still here ?
    # or -> false
    # and -> true
    if condition_operator == 'or':
        return False
    else:
        return True



# -*- coding: utf-8 -*-

# Python2 vs Python3
try:
    from Queue import Queue
except ImportError:
    from queue import Queue

from ScoutSuite.providers.base.configs.base import BaseConfig


class AzureBaseConfig(BaseConfig):

    def __init__(self, thread_config=4, **kwargs):

        super(AzureBaseConfig, self).__init__(thread_config)

    def _is_provider(self, provider_name):
        return provider_name == 'azure'

    def _get_method(self, api_client, target_type, list_method_name):
        """
        Gets the appropriate method, required as each provider may have particularities

        :return:
        """

        target = getattr(api_client, target_type)
        method = getattr(target, list_method_name)

        return method

    def _get_targets(self, response_attribute, api_client, method, list_params, ignore_list_error):
        """
        Fetch the targets, required as each provider may have particularities

        :return:
        """

        targets = []
        response = method(**list_params)
        for i in response:
            targets.append(i)

        return targets



# -*- coding: utf-8 -*-

from ScoutSuite.providers.azure.configs.base import AzureBaseConfig
from ScoutSuite.providers.azure.utils import get_resource_group_name
from msrestazure.azure_exceptions import CloudError

class SQLDatabaseConfig(AzureBaseConfig):
    targets = (
        ('servers', 'Servers', 'list', {}, False),
    )

    def __init__(self, thread_config):

        self.servers = {}
        self.servers_count = 0

        super(SQLDatabaseConfig, self).__init__(thread_config)

    def parse_servers(self, server, params):
        server_dict = {}
        server_dict['id'] = self.get_non_provider_id(server.id)
        server_dict['name'] = server.name
        server_dict['ad_admin_configured'] = self._is_ad_admin_configured(server)
        server_dict['databases'] = self._parse_databases(server)

        self.servers[server_dict['id']] = server_dict

    def _is_ad_admin_configured(self, server):
        return server.azure_ad_admin_settings is not None

    def _parse_databases(self, server):
        databases = {}
        for db in server.databases:
            # Azure automatically creates a reserved read-only 'master' database to manage an SQL server, ignore it:
            if db.name == "master":
                continue

            db_dict = {}
            db_dict['id'] = db.name
            db_dict['auditing_enabled'] = self._is_auditing_enabled(db)
            db_dict['threat_detection_enabled'] = self._is_threat_detection_enabled(db)
            db_dict['transparent_data_encryption_enabled'] = self._is_transparent_data_encryption_enabled(db)
            db_dict['replication_configured'] = self._is_replication_configured(db)
            databases[db.name] = db_dict

        return databases

    def _is_auditing_enabled(self, db):
        return db.auditing_settings.state == "Enabled"

    def _is_threat_detection_enabled(self, db):
        return db.threat_detection_settings.state == "Enabled"

    def _is_transparent_data_encryption_enabled(self, db):
        return db.transparent_data_encryption_settings.status == "Enabled"

    def _is_replication_configured(self, db):
        return len(db.replication_links) > 0

    def _get_targets(self, response_attribute, api_client, method, list_params, ignore_list_error):
        if response_attribute == "Servers":
            return self._get_servers(api_client, method, list_params)
        else:
            return super(SQLDatabaseConfig, self)._get_targets(response_attribute, api_client, method,
                                                               list_params, ignore_list_error)

    def _get_servers(self, api_client, method, list_params):
        servers = []
        servers_raw = method(**list_params)
        for server in servers_raw:
            resource_group_name = get_resource_group_name(server.id)
            setattr(server, "azure_ad_admin_settings",
                    self._get_azure_ad_admin_settings(api_client, resource_group_name, server.name))
            setattr(server, "databases",
                    self._get_databases(api_client, resource_group_name, server.name))
            servers.append(server)

        return servers

    def _get_databases(self, api_client, resource_group_name, server_name):
        databases = []
        databases_raw = api_client.databases.list_by_server(resource_group_name, server_name)
        for db in databases_raw:
            setattr(db, "auditing_settings",
                    self._get_auditing_settings(api_client, resource_group_name, server_name, db.name))
            setattr(db, "threat_detection_settings",
                    self._get_threat_detection_settings(api_client, resource_group_name, server_name, db.name))
            setattr(db, "transparent_data_encryption_settings",
                    self._get_transparent_data_encryption_settings(api_client, resource_group_name, server_name, db.name))
            setattr(db, "replication_links",
                    list(self._get_replication_links(api_client, resource_group_name, server_name, db.name)))
            databases.append(db)

        return databases

    def _get_auditing_settings(self, api_client, resource_group_name, server_name, database_name):
        return api_client.database_blob_auditing_policies.get(resource_group_name, server_name, database_name)

    def _get_threat_detection_settings(self, api_client, resource_group_name, server_name, database_name):
        return api_client.database_threat_detection_policies.get(resource_group_name, server_name, database_name)

    def _get_transparent_data_encryption_settings(self, api_client, resource_group_name, server_name, database_name):
        return api_client.transparent_data_encryptions.get(resource_group_name, server_name, database_name)

    def _get_azure_ad_admin_settings(self, api_client, resource_group_name, server_name):
        try:
            return api_client.server_azure_ad_administrators.get(resource_group_name, server_name)
        except CloudError:  # no ad admin configured returns a 404 error
            return None

    def _get_replication_links(self, api_client, resource_group_name, server_name, database_name):
        return api_client.replication_links.list_by_database(resource_group_name, server_name, database_name)



# -*- coding: utf-8 -*-

from ScoutSuite.providers.gcp.configs.base import GCPBaseConfig

from googleapiclient.errors import HttpError
import json
from opinel.utils.console import printException, printError

class ComputeEngineConfig(GCPBaseConfig):
    targets = (
        ('instances', 'Instances', 'list', {'project': '{{project_placeholder}}', 'zone': '{{zone_placeholder}}'}, False),
        ('snapshots', 'Snapshots', 'list', {'project': '{{project_placeholder}}'}, False),
        ('networks', 'Networks', 'list', {'project': '{{project_placeholder}}'}, False),
        ('subnetworks', 'Subnetworks', 'list', {'project': '{{project_placeholder}}', 'region': '{{region_placeholder}}'}, False),
        ('firewalls', 'Firewalls', 'list', {'project': '{{project_placeholder}}'}, False),
    )

    def __init__(self, thread_config):
        self.library_type = 'api_client_library'

        self.instances = {}
        self.instances_count = 0
        self.snapshots = {}
        self.snapshots_count = 0
        self.networks = {}
        self.networks_count = 0
        self.subnetworks = {}
        self.subnetworks_count = 0
        self.firewalls = {}
        self.firewalls_count = 0

        # TODO figure out why GCP returns errors when running with more then 1 thread (multithreading)
        super(ComputeEngineConfig, self).__init__(thread_config=1)

    # def get_regions(self, client, project):
    #     """
    #     Returns a list of all the regions. Uses an attribute to store the list in order to only run once.
    #     :param client:
    #     :param project:
    #     :return:
    #     """
    #     try:
    #         if self.regions:
    #             return self.regions
    #         else:
    #             regions_list = []
    #             regions = client.regions().list(project=project).execute()['items']
    #             for region in regions:
    #                 regions_list.append(region['name'])
    #             self.regions = regions_list
    #             return regions_list
    #     except HttpError as e:
    #         raise e
    #     except Exception as e:
    #         printException(e)
    #         return None
    #
    # def get_zones(self, client, project):
    #     """
    #     Returns a list of all the zones. Uses an attribute to store the list in order to only run once.
    #     :param client:
    #     :param project:
    #     :return:
    #     """
    #     try:
    #         if self.zones:
    #             return self.zones
    #         else:
    #             zones_list = []
    #             zones = client.zones().list(project=project).execute()['items']
    #             for zone in zones:
    #                 zones_list.append(zone['name'])
    #             self.zones = zones_list
    #             return zones_list
    #     except HttpError as e:
    #         raise e
    #     except Exception as e:
    #         printException(e)
    #         return None

    def parse_instances(self, instance, params):
        instance_dict = {}
        instance_dict['id'] = self.get_non_provider_id(instance['name'])
        instance_dict['project_id'] = instance['selfLink'].split('/')[-5]
        instance_dict['name'] = instance['name']
        instance_dict['description'] = instance['description'] if 'description' in instance and instance['description'] else 'N/A'
        instance_dict['creation_timestamp'] = instance['creationTimestamp']
        instance_dict['zone'] = instance['zone'].split('/')[-1]
        instance_dict['tags'] = instance['tags']
        instance_dict['status'] = instance['status']
        instance_dict['zone_url_'] = instance['zone']
        instance_dict['network_interfaces'] = instance['networkInterfaces']
        instance_dict['service_accounts'] = instance['serviceAccounts']
        instance_dict['deletion_protection_enabled'] = instance['deletionProtection']

        # TODO this should be done in it's own getter method and merged with instances during postprocessing
        instance_dict['disks'] = {}
        if 'disks' in instance:
            for disk in instance['disks']:
                instance_dict['disks'][self.get_non_provider_id(disk['deviceName'])] = {
                    'type': disk['type'],
                    'mode': disk['mode'],
                    'source_url': disk['source'],
                    'source_device_name': disk['deviceName'],
                    'bootable': disk['boot']
                    }

        self.instances[instance_dict['id']] = instance_dict

    def parse_snapshots(self, snapshot, params):
        snapshot_dict = {}
        snapshot_dict['id'] = snapshot['id']
        snapshot_dict['name'] = snapshot['name']
        snapshot_dict['description'] = snapshot['description'] if 'description' in snapshot and snapshot['description'] else 'N/A'
        snapshot_dict['creation_timestamp'] = snapshot['creationTimestamp']
        snapshot_dict['status'] = snapshot['status']
        snapshot_dict['source_disk_id'] = snapshot['sourceDiskId']
        snapshot_dict['source_disk_url'] = snapshot['sourceDisk']
        self.snapshots[snapshot_dict['id']] = snapshot_dict

    def parse_networks(self, network, params):
        network_dict = {}
        network_dict['id'] = network['id']
        network_dict['project_id'] = network['selfLink'].split('/')[-4]
        network_dict['name'] = network['name']
        network_dict['description'] = network['description'] if 'description' in network and network['description'] else 'N/A'
        network_dict['creation_timestamp'] = network['creationTimestamp']
        network_dict['network_url'] = network['selfLink']
        network_dict['subnetwork_urls'] = network.get('subnetworks', None)
        network_dict['auto_subnet'] = network.get('autoCreateSubnetworks',  None)
        network_dict['routing_config'] = network['routingConfig']
        self.networks[network_dict['id']] = network_dict

    def parse_subnetworks(self, subnetwork, params):
        subnetwork_dict = {}
        subnetwork_dict['id'] = subnetwork['id']
        subnetwork_dict['project_id'] = subnetwork['selfLink'].split('/')[-5]
        subnetwork_dict['region'] = subnetwork['region'].split('/')[-1]
        subnetwork_dict['region'] = subnetwork['region'].split('/')[-1]
        subnetwork_dict['name'] = "%s-%s" % (subnetwork['name'], subnetwork_dict['region'])
        subnetwork_dict['subnetwork'] = subnetwork['network'].split('/')[-1]
        subnetwork_dict['gateway_address'] = subnetwork['gatewayAddress']
        subnetwork_dict['ip_range'] = subnetwork['ipCidrRange']
        subnetwork_dict['creation_timestamp'] = subnetwork['creationTimestamp']
        subnetwork_dict['private_ip_google_access'] = subnetwork['privateIpGoogleAccess']
        self.subnetworks[subnetwork_dict['id']] = subnetwork_dict

    def parse_firewalls(self, firewall, params):
        firewall_dict = {}
        firewall_dict['id'] = firewall['id']
        firewall_dict['project_id'] = firewall['selfLink'].split('/')[-4]
        firewall_dict['name'] = firewall['name']
        firewall_dict['description'] = firewall['description'] if \
            'description' in firewall and firewall['description'] else 'N/A'
        firewall_dict['creation_timestamp'] = firewall['creationTimestamp']
        firewall_dict['network'] = firewall['network'].split('/')[-1]
        firewall_dict['network_url'] = firewall['network']
        firewall_dict['priority'] = firewall['priority']
        firewall_dict['source_ranges'] = firewall['sourceRanges'] if 'sourceRanges' in firewall else []
        firewall_dict['source_tags'] = firewall['sourceTags'] if 'sourceTags' in firewall else []
        firewall_dict['target_tags'] = firewall['targetTags'] if 'targetTags' in firewall else []
        firewall_dict['direction'] = firewall['direction']
        firewall_dict['disabled'] = firewall['disabled']

        # Parse FW rules
        for direction in ['allowed', 'denied']:
            direction_string = '%s_traffic' % direction
            firewall_dict[direction_string] = {
                'tcp': [],
                'udp': [],
                'icmp': []
            }
            if direction in firewall:
                firewall_dict['action'] = direction
                for rule in firewall[direction]:
                    if rule['IPProtocol'] not in firewall_dict[direction_string]:
                        firewall_dict[direction_string][rule['IPProtocol']] = ['*']
                    else:
                        if rule['IPProtocol'] == 'all':
                            for protocol in firewall_dict[direction_string]:
                                firewall_dict[direction_string][protocol] = ['0-65535']
                            break
                        else:
                            if firewall_dict[direction_string][rule['IPProtocol']] != ['0-65535']:
                                if 'ports' in rule:
                                    firewall_dict[direction_string][rule['IPProtocol']] += rule['ports']
                                else:
                                    firewall_dict[direction_string][rule['IPProtocol']] = ['0-65535']

        self.firewalls[firewall_dict['id']] = firewall_dict



# -*- coding: utf-8 -*-
from __future__ import print_function

import os
import sys

from opinel.utils.console import printError

from six.moves import input

from ScoutSuite import AWSCONFIG, EXCEPTIONS, HTMLREPORT, AWSRULESET, AWSCONFIG_FILE, EXCEPTIONS_FILE, HTMLREPORT_FILE, AWSRULESET_FILE


def prompt_4_yes_no(question):
    """
    Ask a question and prompt for yes or no

    :param question:                    Question to ask; answer is yes/no
    :return:                            :boolean
    """

    while True:
        sys.stdout.write(question + ' (y/n)? ')
        choice = input().lower()
        if choice == 'yes' or choice == 'y':
            return True
        elif choice == 'no' or choice == 'n':
            return False
        else:
            printError('\'%s\' is not a valid answer. Enter \'yes\'(y) or \'no\'(n).' % choice)


def prompt_4_overwrite(filename, force_write):
    """
    Confirm before overwriting existing files. Do not prompt if the file does not exist or force_write is set

    :param filename:                    Name of the file to be overwritten
    :param force_write:                 Do not ask for confirmation and automatically return True if set
    :return:                            :boolean
    """
    #
    if not os.path.exists(filename) or force_write:
        return True
    return prompt_4_yes_no('File \'{}\' already exists. Do you want to overwrite it'.format(filename))


def get_filename(config_type, profile, report_dir):
        if config_type == AWSCONFIG:
            filename = AWSCONFIG_FILE
            first_line = 'scoutsuite_results ='
        elif config_type == EXCEPTIONS:
            filename = EXCEPTIONS_FILE
            first_line = 'exceptions ='
        elif config_type == HTMLREPORT:
            filename = HTMLREPORT_FILE
            first_line = None
        elif config_type == AWSRULESET:
            filename = AWSRULESET_FILE
            first_line = 'scoutsuite_results ='
        else:
            printError('invalid config type provided (%s)' % config_type)
            raise Exception
        # Append profile name if necessary
        if profile != 'default' and config_type != AWSRULESET:
            name, extention = filename.split('.')
            filename = '%s-%s.%s' % (name, profile, extention)
        return (os.path.join(report_dir, filename), first_line)



# -*- coding: utf-8 -*-

from opinel.utils.aws import handle_truncated_response
from opinel.utils.console import printDebug, printInfo


def get_organization_account_ids(api_client, exceptions = [], quiet = True):

    # List all accounts in the organization
    org_accounts = get_organization_accounts(api_client, exceptions, quiet)
    return [ account['Id'] for account in org_accounts ]


def get_organization_accounts(api_client, exceptions = [], quiet = True):

    # List all accounts in the organization
    org_accounts = handle_truncated_response(api_client.list_accounts, {}, ['Accounts'])['Accounts']
    if not quiet:
        printInfo('Found %d accounts in the organization.' % len(org_accounts))
        for account in org_accounts:
            printDebug(str(account))
    if len(exceptions):
        filtered_accounts = []
        for account in org_accounts:
            if account['Id'] not in exceptions:
                filtered_accounts.append(account)
        org_accounts = filtered_accounts
    return org_accounts


def get_organizational_units(api_client):
    ous = []
    roots = api_client.list_roots()['Roots']
    return get_children_organizational_units(api_client, roots)


def get_children_organizational_units(api_client, parents):
    ous = []
    for parent in parents:
        children = handle_truncated_response(api_client.list_organizational_units_for_parent, {'ParentId': parent['Id']}, ['OrganizationalUnits'])['OrganizationalUnits']
        if len(children):
            ous += get_children_organizational_units(api_client, children)
        else:
            ous.append(parent)
    return ous

def list_accounts_for_parent(api_client, parent):
    return handle_truncated_response(api_client.list_accounts_for_parent, {'ParentId': parent['Id']}, ['Accounts'])['Accounts']




# -*- coding: utf-8 -*-

import re

from opinel.utils.aws import handle_truncated_response
from opinel.utils.credentials import generate_password
from opinel.utils.console import printInfo, printError, printException



def add_user_to_group(iam_client, user, group, quiet = False):
    """
    Add an IAM user to an IAM group

    :param iam_client:
    :param group:
    :param user:
    :param user_info:
    :param dry_run:
    :return:
    """
    if not quiet:
        printInfo('Adding user to group %s...' % group)
    iam_client.add_user_to_group(GroupName = group, UserName = user)


def create_groups(iam_client, groups):
    """
    Create a number of IAM group, silently handling exceptions when entity already exists
                                        .
    :param iam_client:                  AWS API client for IAM
    :param groups:                      Name of IAM groups to be created.

    :return:                            None
    """
    groups_data = []
    if type(groups) != list:
        groups = [ groups ]
    for group in groups:
        errors = []
        try:
            printInfo('Creating group %s...' % group)
            iam_client.create_group(GroupName = group)
        except  Exception as e:
            if e.response['Error']['Code'] != 'EntityAlreadyExists':
                printException(e)
                errors.append('iam:creategroup')
        groups_data.append({'groupname': group, 'errors': errors})
    return groups_data


def create_user(iam_client, user, groups = [], with_password= False, with_mfa = False, with_access_key = False, require_password_reset = True):
    """

    :param iam_client:                  AWS API client for IAM
    :param user:                        Name of the user to create
    :param groups:                      Name of the IAM groups to add the user to
    :param with_password:               Boolean indicating whether creation of a password should be done
    :param with_mfa:                    Boolean indicating whether creation of an MFA device should be done
    :param with_access_key:             Boolean indicating whether creation of an API access key should be done
    :param require_password_reset:      Boolean indicating whether users should reset their password after first login
    :return:
    """
    user_data = {'username': user, 'errors': []}
    printInfo('Creating user %s...' % user)
    try:
        iam_client.create_user(UserName = user)
    except Exception as e:
        user_data['errors'].append('iam:createuser')
        return user_data
    # Add user to groups
    if type(groups) != list:
        groups = [ groups ]
    for group in groups:
        try:
            add_user_to_group(iam_client, user, group)
        except Exception as e:
            printException(e)
            user_data['errors'].append('iam:addusertogroup - %s' % group)
    # Generate password
    if with_password:
        try:
            printInfo('Creating a login profile...')
            user_data['password'] = generate_password()
            iam_client.create_login_profile(UserName = user, Password = user_data['password'] , PasswordResetRequired = require_password_reset)
        except Exception as e:
            printException(e)
            user_data['errors'].append('iam:createloginprofile')
    # Enable MFA
    if False and with_mfa:
        printInfo('Enabling MFA...')
        serial = ''
        mfa_code1 = ''
        mfa_code2 = ''
        # Create an MFA device, Display the QR Code, and activate the MFA device
        try:
            mfa_serial = False # enable_mfa(iam_client, user, '%s/qrcode.png' % user)
        except Exception as e:
            return 42
    # Request access key
    if with_access_key:
        try:
            printInfo('Creating an API access key...')
            access_key = iam_client.create_access_key(UserName=user)['AccessKey']
            user_data['AccessKeyId'] = access_key['AccessKeyId']
            user_data['SecretAccessKey'] = access_key['SecretAccessKey']
        except Exception as e:
            printException(e)
            user_data['errors'].append('iam:createaccesskey')
    return user_data


def delete_user(iam_client, user, mfa_serial = None, keep_user = False, terminated_groups = []):
    """
    Delete IAM user

    :param iam_client:
    :param user:
    :param mfa_serial:
    :param keep_user:
    :param terminated_groups:
    :return:
    """
    errors = []
    printInfo('Deleting user %s...' % user)
    # Delete access keys
    try:
        aws_keys = get_access_keys(iam_client, user)
        for aws_key in aws_keys:
            try:
                printInfo('Deleting access key ID %s... ' % aws_key['AccessKeyId'], False)
                iam_client.delete_access_key(AccessKeyId = aws_key['AccessKeyId'], UserName = user)
                printInfo('Success')
            except Exception as e:
                printInfo('Failed')
                printException(e)
                errors.append(e.response['Error']['Code'])
    except Exception as e:
        printException(e)
        printError('Failed to get access keys for user %s.' % user)
    # Deactivate and delete MFA devices
    try:
        mfa_devices = iam_client.list_mfa_devices(UserName = user)['MFADevices']
        for mfa_device in mfa_devices:
            serial = mfa_device['SerialNumber']
            try:
                printInfo('Deactivating MFA device %s... ' % serial, False)
                iam_client.deactivate_mfa_device(SerialNumber = serial, UserName = user)
                printInfo('Success')
            except Exception as e:
                printInfo('Failed')
                printException(e)
                errors.append(e.response['Error']['Code'])
            delete_virtual_mfa_device(iam_client, serial)
        if mfa_serial:
            delete_virtual_mfa_device(iam_client, mfa_serial)
    except Exception as e:
        printException(e)
        printError('Faile to fetch/delete MFA device serial number for user %s.' % user)
        errors.append(e.response['Error']['Code'])
    # Remove IAM user from groups
    try:
        groups = iam_client.list_groups_for_user(UserName = user)['Groups']
        for group in groups:
            try:
                printInfo('Removing from group %s... ' % group['GroupName'], False)
                iam_client.remove_user_from_group(GroupName = group['GroupName'], UserName = user)
                printInfo('Success')
            except Exception as e:
                printInfo('Failed')
                printException(e)
                errors.append(e.response['Error']['Code'])
    except Exception as e:
        printException(e)
        printError('Failed to fetch IAM groups for user %s.' % user)
        errors.append(e.response['Error']['Code'])
    # Delete login profile
    login_profile = []
    try:
        login_profile = iam_client.get_login_profile(UserName = user)['LoginProfile']
    except Exception as e:
        pass
    try:
        if len(login_profile):
            printInfo('Deleting login profile... ', False)
            iam_client.delete_login_profile(UserName = user)
            printInfo('Success')
    except Exception as e:
        printInfo('Failed')
        printException(e)
        errors.append(e.response['Error']['Code'])
    # Delete inline policies
    try:
        printInfo('Deleting inline policies... ', False)
        policies = iam_client.list_user_policies(UserName = user)
        for policy in policies['PolicyNames']:
            iam_client.delete_user_policy(UserName = user, PolicyName = policy)
        printInfo('Success')
    except Exception as e:
        printInfo('Failed')
        printException(e)
        errors.append(e.response['Error']['Code'])
    # Detach managed policies
    try:
        printInfo('Detaching managed policies... ', False)
        policies = iam_client.list_attached_user_policies(UserName = user)
        for policy in policies['AttachedPolicies']:
            iam_client.detach_user_policy(UserName = user, PolicyArn = policy['PolicyArn'])
        printInfo('Success')
    except Exception as e:
        printInfo('Failed')
        printException(e)
        errors.append(e.response['Error']['Code'])
    # Delete IAM user
    try:
        if not keep_user:
            iam_client.delete_user(UserName = user)
            printInfo('User %s deleted.' % user)
        else:
            for group in terminated_groups:
                add_user_to_group(iam_client, group, user)
    except Exception as e:
        printException(e)
        printError('Failed to delete user.')
        errors.append(e.response['Error']['Code'])
        pass
    return errors


def delete_virtual_mfa_device(iam_client, mfa_serial):
    """
    Delete a vritual MFA device given its serial number

    :param iam_client:
    :param mfa_serial:
    :return:
    """
    try:
        printInfo('Deleting MFA device %s...' % mfa_serial)
        iam_client.delete_virtual_mfa_device(SerialNumber = mfa_serial)
    except Exception as e:
        printException(e)
        printError('Failed to delete MFA device %s' % mfa_serial)
        pass

def get_access_keys(iam_client, user_name):
    """

    :param iam_client:
    :param user_name:
    :return:
    """
    keys = handle_truncated_response(iam_client.list_access_keys, {'UserName': user_name}, ['AccessKeyMetadata'])['AccessKeyMetadata']
    return keys


def init_group_category_regex(category_groups, category_regex_args):
    """
    Initialize and compile regular expression for category groups

    :param category_regex_args:         List of string regex

    :return:                            List of compiled regex
    """
    category_regex = []
    authorized_empty_regex = 1
    if len(category_regex_args) and len(category_groups) != len(category_regex_args):
        printError('Error: you must provide as many regex as category groups.')
        return None
    for regex in category_regex_args:
        if len(regex) < 1:
            if authorized_empty_regex > 0:
                category_regex.append(None)
                authorized_empty_regex -= 1
            else:
                printError('Error: you cannot have more than one empty regex to automatically assign groups to users.')
                return None
        else:
            category_regex.append(re.compile(regex))
    return category_regex



def show_access_keys(iam_client, user_name):
    """

    :param iam_client:
    :param user_name:
    :return:
    """
    keys = get_access_keys(iam_client, user_name)
    printInfo('User \'%s\' currently has %s access keys:' % (user_name, len(keys)))
    for key in keys:
        printInfo('\t%s (%s)' % (key['AccessKeyId'], key['Status']))



# -*- coding: utf-8 -*-

from ScoutSuite.providers.gcp.configs.base import GCPBaseConfig

from opinel.utils.console import printError, printException, printInfo

from googleapiclient import discovery
from ScoutSuite.providers.gcp.utils import gcp_connect_service


class IAMConfig(GCPBaseConfig):
    targets = (
        ('projects.serviceAccounts', 'Service Accounts', 'list', {'name': 'projects/{{project_placeholder}}'}, False),
    )

    def __init__(self, thread_config):

        self.library_type = 'api_client_library'

        self.service_accounts = {}
        self.service_accounts_count = 0

        super(IAMConfig, self).__init__(thread_config)

    def parse_projects_serviceAccounts(self, service_account, params):

        service_account_dict = {}

        service_account_dict['id'] = service_account['uniqueId']
        service_account_dict['display_name'] = service_account['displayName'] if 'displayName' in service_account else 'N/A'
        service_account_dict['name'] = service_account['email']
        service_account_dict['email'] = service_account['email']
        service_account_dict['project_id'] = service_account['projectId']

        keys = self._get_service_account_keys(params['api_client'],
                                              service_account_dict['project_id'],
                                              service_account_dict['email'])
        service_account_dict['keys'] = {}
        if keys:
            for key in keys:
                service_account_dict['keys'][key['name'].split('/')[-1]] = {
                    'valid_after': key['validAfterTime'],
                    'valid_before': key['validBeforeTime'],
                    'key_algorithm': key['keyAlgorithm']
                }

        bindings = self._get_service_account_iam_policy(params['api_client'],
                                                        service_account_dict['project_id'],
                                                        service_account_dict['email'])
        service_account_dict['bindings'] = []
        if bindings:
            service_account_dict['bindings'] = bindings


        self.service_accounts[service_account_dict['id']] = service_account_dict

        # required as target is 'projects.serviceAccounts' and not 'service_accounts'
        self.service_accounts_count+=1


    def _get_service_account_keys(self, api_client, project_id, service_account_email):

        try:
            #FIXME for some reason using the api_client fails, creating a new client doesn't generate an error...
            client = discovery.build('iam', 'v1', cache_discovery=False)
            # client = gcp_connect_service(service='iam')

            # response = api_client.projects().serviceAccounts().keys().list(
            response = client.projects().serviceAccounts().keys().list(
                name='projects/%s/serviceAccounts/%s' % (project_id, service_account_email)).execute()
            if 'keys' in response:
                return response['keys']
            else:
                return None
        except Exception as e:
            printError('Failed to get keys for service account %s: %s' % (service_account_email, e))
            return None

    def _get_service_account_iam_policy(self, api_client, project_id, service_account_email):

        try:
            #FIXME for some reason using the api_client fails, creating a new client doesn't generate an error...
            client = discovery.build('iam', 'v1')

            # response = api_client.projects().serviceAccounts().getIamPolicy(
            response = client.projects().serviceAccounts().getIamPolicy(
                resource='projects/%s/serviceAccounts/%s' % (project_id, service_account_email)).execute()
            if 'bindings' in response:
                return response['bindings']
            else:
                return None
        except Exception as e:
            printError('Failed to get IAM policy for service account %s: %s' % (service_account_email, e))
            return None



# -*- coding: utf-8 -*-
"""
EC2-related classes and functions
"""

# TODO: move a lot of this to VPCconfig, and use some sort of filter to only list SGs in EC2 classic
import netaddr
import base64

from opinel.utils.aws import get_name
from opinel.utils.console import printException, printInfo
from opinel.utils.fs import load_data
from opinel.utils.globals import manage_dictionary

from ScoutSuite.providers.aws.configs.vpc import VPCConfig
from ScoutSuite.providers.base.configs.browser import get_attribute_at
from ScoutSuite.providers.base.provider import BaseProvider
from ScoutSuite.utils import get_keys, ec2_classic
from ScoutSuite.providers.aws.configs.regions import RegionalServiceConfig, RegionConfig, api_clients


########################################
# Globals
########################################

icmp_message_types_dict = load_data('icmp_message_types.json', 'icmp_message_types')
protocols_dict = load_data('protocols.json', 'protocols')


########################################
# EC2RegionConfig
########################################

class EC2RegionConfig(RegionConfig):
    """
    EC2 configuration for a single AWS region
    """

    def parse_elastic_ip(self, global_params, region, eip):
        """

        :param global_params:
        :param region:
        :param eip:
        :return:
        """
        self.elastic_ips[eip['PublicIp']] = eip

    def parse_instance(self, global_params, region, reservation):
        """
        Parse a single EC2 instance

        :param global_params:           Parameters shared for all regions
        :param region:                  Name of the AWS region
        :param instance:                Cluster
        """
        for i in reservation['Instances']:
            instance = {}
            vpc_id = i['VpcId'] if 'VpcId' in i and i['VpcId'] else ec2_classic
            manage_dictionary(self.vpcs, vpc_id, VPCConfig(self.vpc_resource_types))
            instance['reservation_id'] = reservation['ReservationId']
            instance['id'] = i['InstanceId']
            instance['monitoring_enabled'] = i['Monitoring']['State'] == 'enabled'
            instance['user_data'] = self._get_user_data(region, instance['id'])
            get_name(i, instance, 'InstanceId')
            get_keys(i, instance, ['KeyName', 'LaunchTime', 'InstanceType', 'State', 'IamInstanceProfile', 'SubnetId'])
            # Network interfaces & security groups
            manage_dictionary(instance, 'network_interfaces', {})
            for eni in i['NetworkInterfaces']:
                nic = {}
                get_keys(eni, nic, ['Association', 'Groups', 'PrivateIpAddresses', 'SubnetId', 'Ipv6Addresses'])
                instance['network_interfaces'][eni['NetworkInterfaceId']] = nic
            self.vpcs[vpc_id].instances[i['InstanceId']] = instance

    def _get_user_data(self, region, instance_id):
        user_data_response = api_clients[region].describe_instance_attribute(Attribute='userData', InstanceId=instance_id)

        if 'Value' not in user_data_response['UserData'].keys():
            return None

        return base64.b64decode(user_data_response['UserData']['Value']).decode('utf-8')

    def parse_image(self, global_params, region, image):
        """
        Parses a single AMI (Amazon Machine Image)

        :param global_params:           Parameters shared for all regions
        :param region:                  Name of the AWS region
        :param snapshot:                Single image
        """
        id = image['ImageId']
        name = image['Name']

        image['id'] = id
        image['name'] = name

        self.images[id] = image 

    def parse_security_group(self, global_params, region, group):
        """
        Parse a single Redsfhit security group

        :param global_params:           Parameters shared for all regions
        :param region:                  Name of the AWS region
        :param security)_group:         Security group
        """
        vpc_id = group['VpcId'] if 'VpcId' in group and group['VpcId'] else ec2_classic
        manage_dictionary(self.vpcs, vpc_id, VPCConfig(self.vpc_resource_types))
        security_group = {}
        security_group['name'] = group['GroupName']
        security_group['id'] = group['GroupId']
        security_group['description'] = group['Description']
        security_group['owner_id'] = group['OwnerId']
        security_group['rules'] = {'ingress': {}, 'egress': {}}
        security_group['rules']['ingress']['protocols'], security_group['rules']['ingress'][
            'count'] = self.__parse_security_group_rules(group['IpPermissions'])
        security_group['rules']['egress']['protocols'], security_group['rules']['egress'][
            'count'] = self.__parse_security_group_rules(group['IpPermissionsEgress'])
        self.vpcs[vpc_id].security_groups[group['GroupId']] = security_group

    def __parse_security_group_rules(self, rules):
        """

        :param self:
        :param rules:
        :return:
        """
        protocols = {}
        rules_count = 0
        for rule in rules:
            ip_protocol = rule['IpProtocol'].upper()
            if ip_protocol == '-1':
                ip_protocol = 'ALL'
            protocols = manage_dictionary(protocols, ip_protocol, {})
            protocols[ip_protocol] = manage_dictionary(protocols[ip_protocol], 'ports', {})
            # Save the port (single port or range)
            port_value = 'N/A'
            if 'FromPort' in rule and 'ToPort' in rule:
                if ip_protocol == 'ICMP':
                    # FromPort with ICMP is the type of message
                    port_value = icmp_message_types_dict[str(rule['FromPort'])]
                elif rule['FromPort'] == rule['ToPort']:
                    port_value = str(rule['FromPort'])
                else:
                    port_value = '%s-%s' % (rule['FromPort'], rule['ToPort'])
            manage_dictionary(protocols[ip_protocol]['ports'], port_value, {})
            # Save grants, values are either a CIDR or an EC2 security group
            for grant in rule['UserIdGroupPairs']:
                manage_dictionary(protocols[ip_protocol]['ports'][port_value], 'security_groups', [])
                protocols[ip_protocol]['ports'][port_value]['security_groups'].append(grant)
                rules_count = rules_count + 1
            for grant in rule['IpRanges']:
                manage_dictionary(protocols[ip_protocol]['ports'][port_value], 'cidrs', [])
                protocols[ip_protocol]['ports'][port_value]['cidrs'].append({'CIDR': grant['CidrIp']})
                rules_count = rules_count + 1
            # IPv6
            for grant in rule['Ipv6Ranges']:
                manage_dictionary(protocols[ip_protocol]['ports'][port_value], 'cidrs', [])
                protocols[ip_protocol]['ports'][port_value]['cidrs'].append({'CIDR': grant['CidrIpv6']})
                rules_count = rules_count + 1

        return protocols, rules_count

    def parse_snapshot(self, global_params, region, snapshot):
        """

        :param global_params:           Parameters shared for all regions
        :param region:                  Name of the AWS region
        :param snapshot:                  Single snapshot
        :return:
        """
        snapshot['id'] = snapshot.pop('SnapshotId')
        snapshot['name'] = get_name(snapshot, snapshot, 'id')
        self.snapshots[snapshot['id']] = snapshot
        # Get snapshot attribute
        snapshot['createVolumePermission'] = \
        api_clients[region].describe_snapshot_attribute(Attribute='createVolumePermission', SnapshotId=snapshot['id'])[
            'CreateVolumePermissions']
        snapshot['public'] = self._is_public(snapshot)

    def _is_public(self, snapshot):
        return any([permission.get('Group') == 'all' for permission in snapshot['createVolumePermission']])

    def parse_volume(self, global_params, region, volume):
        """

        :param global_params:           Parameters shared for all regions
        :param region:                  Name of the AWS region
        :param volume:                  Single EBS volume
        :return:
        """
        volume['id'] = volume.pop('VolumeId')
        volume['name'] = get_name(volume, volume, 'id')
        self.volumes[volume['id']] = volume


########################################
# EC2Config
########################################

class EC2Config(RegionalServiceConfig):
    """
    EC2 configuration for all AWS regions
    """

    region_config_class = EC2RegionConfig

    def __init__(self, service_metadata, thread_config=4):
        super(EC2Config, self).__init__(service_metadata, thread_config)


########################################
##### EC2 analysis functions
########################################

def analyze_ec2_config(ec2_info, aws_account_id, force_write):
    try:
        printInfo('Analyzing EC2 config... ', newLine=False)
        # Custom EC2 analysis
        #        check_for_elastic_ip(ec2_info)
        # FIXME - commented for now as this method doesn't seem to be defined anywhere'
        # list_network_attack_surface(ec2_info, 'attack_surface', 'PublicIpAddress')
        # TODO: make this optional, commented out for now
        # list_network_attack_surface(ec2_info, 'private_attack_surface', 'PrivateIpAddress')
        printInfo('Success')
    except Exception as e:
        printInfo('Error')
        printException(e)

def add_security_group_name_to_ec2_grants_callback(ec2_config, current_config, path, current_path, ec2_grant,
                                                   callback_args):
    """
    Callback

    :param ec2_config:
    :param current_config:
    :param path:
    :param current_path:
    :param ec2_grant:
    :param callback_args:
    :return:
    """
    sg_id = ec2_grant['GroupId']
    if sg_id in current_path:
        target = current_path[:(current_path.index(sg_id) + 1)]
        ec2_grant['GroupName'] = get_attribute_at(ec2_config, target, 'name')
    elif ec2_grant['UserId'] == callback_args['AWSAccountId']:
        if 'VpcId' in ec2_grant:
            target = current_path[:(current_path.index('vpcs') + 1)]
            target.append(ec2_grant['VpcId'])
            target.append('security_groups')
            target.append(sg_id)
        else:
            target = current_path[:(current_path.index('security_groups') + 1)]
            target.append(sg_id)
        ec2_grant['GroupName'] = get_attribute_at(ec2_config, target, 'name')


def check_for_elastic_ip(ec2_info):
    """
    Check that the whitelisted EC2 IP addresses are not static IPs

    :param ec2_info:
    :return:
    """
    # Build a list of all elatic IP in the account
    elastic_ips = []
    for region in ec2_info['regions']:
        if 'elastic_ips' in ec2_info['regions'][region]:
            for eip in ec2_info['regions'][region]['elastic_ips']:
                elastic_ips.append(eip)
    new_items = []
    new_macro_items = []
    for i, item in enumerate(ec2_info['violations']['non-elastic-ec2-public-ip-whitelisted'].items):
        ip = netaddr.IPNetwork(item)
        found = False
        for eip in elastic_ips:
            eip = netaddr.IPNetwork(eip)
            if ip in eip:
                found = True
                break
        if not found:
            new_items.append(ec2_info['violations']['non-elastic-ec2-public-ip-whitelisted'].items[i])
            new_macro_items.append(ec2_info['violations']['non-elastic-ec2-public-ip-whitelisted'].macro_items[i])
    ec2_info['violations']['non-elastic-ec2-public-ip-whitelisted'].items = new_items
    ec2_info['violations']['non-elastic-ec2-public-ip-whitelisted'].macro_items = new_macro_items


def link_elastic_ips_callback2(ec2_config, current_config, path, current_path, instance_id, callback_args):
    if instance_id == callback_args['instance_id']:
        if not 'PublicIpAddress' in current_config:
            current_config['PublicIpAddress'] = callback_args['elastic_ip']
        elif current_config['PublicIpAddress'] != callback_args['elastic_ip']:
            printInfo('Warning: public IP address exists (%s) for an instance associated with an elastic IP (%s)' % (
            current_config['PublicIpAddress'], callback_args['elastic_ip']))
            # This can happen... fix it


def list_instances_in_security_groups(region_info):
    """
    Once all the data has been fetched, iterate through instances and list them
    Could this be done when all the "used_by" values are set ??? TODO

    :param region_info:
    :return:
    """
    for vpc in region_info['vpcs']:
        if not 'instances' in region_info['vpcs'][vpc]:
            return
        for instance in region_info['vpcs'][vpc]['instances']:
            state = region_info['vpcs'][vpc]['instances'][instance]['State']['Name']
            for sg in region_info['vpcs'][vpc]['instances'][instance]['security_groups']:
                sg_id = sg['GroupId']
                manage_dictionary(region_info['vpcs'][vpc]['security_groups'][sg_id], 'instances', {})
                manage_dictionary(region_info['vpcs'][vpc]['security_groups'][sg_id]['instances'], state, [])
                region_info['vpcs'][vpc]['security_groups'][sg_id]['instances'][state].append(instance)


def manage_vpc(vpc_info, vpc_id):
    """
    Ensure name and ID are set

    :param vpc_info:
    :param vpc_id:
    :return:
    """
    manage_dictionary(vpc_info, vpc_id, {})
    vpc_info[vpc_id]['id'] = vpc_id
    if not 'name' in vpc_info[vpc_id]:
        vpc_info[vpc_id]['name'] = vpc_id



# -*- coding: utf-8 -*-

import copy

from threading import Thread

# Python2 vs Python3
try:
    from Queue import Queue
except ImportError:
    from queue import Queue

from hashlib import sha1

from ScoutSuite.providers.base.configs.threads import thread_configs

# TODO do this better without name conflict
from opinel.utils.aws import connect_service
from ScoutSuite.providers.gcp.utils import gcp_connect_service
from ScoutSuite.providers.azure.utils import azure_connect_service

from opinel.utils.aws import build_region_list
from opinel.utils.console import printException, printInfo

from ScoutSuite.output.console import FetchStatusLogger
from ScoutSuite.utils import format_service_name

# TODO global is trash
status = None
formatted_string = None


class BaseConfig(object):

    def __init__(self, thread_config=4, **kwargs):
        """

        :param thread_config:
        """

        self.library_type = None if not hasattr(self, 'library_type') else self.library_type

        self.service = type(self).__name__.replace('Config',
                                                   '').lower()  # TODO: use regex with EOS instead of plain replace
        self.thread_config = thread_configs[thread_config]

        # Booleans that define if threads should keep running
        self.run_service_threads = True
        self.run_target_threads = True

    def _is_provider(self, provider_name):
        return False

    def get_non_provider_id(self, name):
        """
        Not all AWS resources have an ID and some services allow the use of "." in names, which break's Scout2's
        recursion scheme if name is used as an ID. Use SHA1(name) instead.

        :param name:                    Name of the resource to
        :return:                        SHA1(name)
        """
        m = sha1()
        m.update(name.encode('utf-8'))
        return m.hexdigest()

    def fetch_all(self, credentials, regions=None, partition_name='aws', targets=None):
        """
        :param credentials:             F
        :param service:                 Name of the service
        :param regions:                 Name of regions to fetch data from
        :param partition_name:          AWS partition to connect to
        :param targets:                 Type of resources to be fetched; defaults to all.
        :return:
        """
        regions = [] if regions is None else regions
        global status, formatted_string

        # Initialize targets
        if not targets:
            targets = type(self).targets
        printInfo('Fetching %s config...' % format_service_name(self.service))
        formatted_string = None

        # FIXME the below should be in moved to each provider's code

        # Connect to the service
        if self._is_provider('aws'):
            if self.service in ['s3']:  # S3 namespace is global but APIs aren't....
                api_clients = {}
                for region in build_region_list(self.service, regions, partition_name):
                    api_clients[region] = connect_service('s3', credentials, region, silent=True)
                api_client = api_clients[list(api_clients.keys())[0]]
            elif self.service == 'route53domains':
                api_client = connect_service(self.service, credentials, 'us-east-1',
                                             silent=True)  # TODO: use partition's default region
            else:
                api_client = connect_service(self.service, credentials, silent=True)

        elif self._is_provider('gcp'):
            api_client = gcp_connect_service(service=self.service, credentials=credentials)

        elif self._is_provider('azure'):
            api_client = azure_connect_service(service=self.service, credentials=credentials)

        # Threading to fetch & parse resources (queue consumer)
        params = {'api_client': api_client}

        if self._is_provider('aws'):
            if self.service in ['s3']:
                params['api_clients'] = api_clients

        # Threading to parse resources (queue feeder)
        target_queue = self._init_threading(self.__fetch_target, params, self.thread_config['parse'])

        # Threading to list resources (queue feeder)
        params = {'api_client': api_client, 'q': target_queue}

        if self._is_provider('aws'):
            if self.service in ['s3']:
                params['api_clients'] = api_clients

        service_queue = self._init_threading(self.__fetch_service, params, self.thread_config['list'])

        # Init display
        self.fetchstatuslogger = FetchStatusLogger(targets)

        # Go
        for target in targets:
            service_queue.put(target)

        # Blocks until all items in the queue have been gotten and processed.
        service_queue.join()
        target_queue.join()

        # Show completion and force newline
        if self._is_provider('aws'):
            # Show completion and force newline
            if self.service != 'iam':
                self.fetchstatuslogger.show(True)
        else:
            self.fetchstatuslogger.show(True)

        # Threads should stop running as queues are empty
        self.run_target_threads = False
        self.run_service_threads = False
        # Put x items in the queues to ensure threads run one last time (and exit)
        for i in range(self.thread_config['parse']):
            target_queue.put(None)
        for j in range(self.thread_config['list']):
            service_queue.put(None)

    def __fetch_target(self, q, params):
        global status
        try:
            while self.run_target_threads:
                try:
                    target_type, target = q.get() or (None, None)
                    if target_type and target:
                        # Make a full copy of the target in case we need to re-queue it
                        backup = copy.deepcopy(target)
                        method = getattr(self, 'parse_%s' %
                                         target_type.replace('.', '_'))  # TODO fix this, hack for GCP API Client libs
                        method(target, params)
                        self.fetchstatuslogger.counts[target_type]['fetched'] += 1
                        self.fetchstatuslogger.show()
                except Exception as e:
                    if hasattr(e, 'response') and \
                            'Error' in e.response and \
                            e.response['Error']['Code'] in ['Throttling']:
                        q.put((target_type, backup), )
                    else:
                        printException(e)
                finally:
                    q.task_done()
        except Exception as e:
            printException(e)
            pass

    def __fetch_service(self, q, params):
        api_client = params['api_client']
        try:
            while self.run_service_threads:
                try:
                    target_type, response_attribute, list_method_name, list_params, ignore_list_error = q.get() or (None, None, None, None, None)

                    if target_type:

                        if not list_method_name:
                            continue

                        try:
                            method = self._get_method(api_client, target_type, list_method_name)
                        except Exception as e:
                            printException(e)
                            continue

                        try:
                            targets = self._get_targets(response_attribute, api_client, method, list_params, ignore_list_error)
                        except Exception as e:
                            if not ignore_list_error:
                                printException(e)
                            targets = []

                        self.fetchstatuslogger.counts[target_type]['discovered'] += len(targets)

                        for target in targets:
                            params['q'].put((target_type, target), )

                except Exception as e:
                    printException(e)
                finally:
                    q.task_done()
        except Exception as e:
            printException(e)
            pass

    def _get_method(self, api_client, target_type, list_method_name):
        """
        Gets the appropriate method, required as each provider may have particularities

        :return:
        """
        return None

    def _get_targets(self, response_attribute, api_client, method, list_params, ignore_list_error):
        """
        Gets the targets, required as each provider may have particularities

        :return:
        """
        return None

    def finalize(self):
        for t in self.fetchstatuslogger.counts:
            setattr(self, '%s_count' % t, self.fetchstatuslogger.counts[t]['fetched'])
        self.__delattr__('fetchstatuslogger')

    def _init_threading(self, function, params=None, num_threads=10):
        params = {} if params is None else params
        # Init queue and threads
        q = Queue(maxsize=0)  # TODO: find something appropriate
        for i in range(num_threads):
            worker = Thread(target=function, args=(q, params))
            worker.setDaemon(True)
            worker.start()
        return q



# -*- coding: utf-8 -*-

from ScoutSuite.providers.gcp.configs.base import GCPBaseConfig

from opinel.utils.console import printError, printException, printInfo


class CloudResourceManager(GCPBaseConfig):
    targets = (
        ('projects', 'Bindings', 'getIamPolicy', {'resource': '{{project_placeholder}}'}, False),
    )

    def __init__(self, thread_config):

        self.library_type = 'api_client_library'

        self.bindings = {}
        self.bindings_count = 0

        super(CloudResourceManager, self).__init__(thread_config)

    # this is a misnamer, as the method returns IAM policies.
    # In addition, the 'project' method already exists for the BaseConfig object which creates a conflict.
    # TODO solve this if the Cloud Resource Manager needs to be used to get multiple things
    def parse_projects(self, binding, params):
        """
        """

        binding_dict = {}
        binding_dict['id'] = self.get_non_provider_id(binding['role'])
        binding_dict['name'] = binding['role'].split('/')[-1]

        binding_dict['members'] = {'users': [],
                                   'groups': [],
                                   'service_accounts': []}
        if 'members' in binding:
            for member in binding['members']:
                type = member.split(':')[0]
                entity = member.split(':')[1]
                if type == 'user':
                    binding_dict['members']['users'].append(entity)
                elif type == 'group':
                    binding_dict['members']['groups'].append(entity)
                elif type == 'serviceAccount':
                    binding_dict['members']['service_accounts'].append(entity)
                else:
                    printException('Type %s not handled' % type)

        self.bindings[binding_dict['id']] = binding_dict

        # required as target is 'projects' and not 'bindings'
        self.bindings_count+=1



# -*- coding: utf-8 -*-

import copy
import os
import sys

try:
    from opinel.utils.aws import get_aws_account_id, get_partition_name
    from opinel.utils.console import configPrintException, printInfo, printDebug
    from opinel.utils.credentials import read_creds
    from opinel.utils.globals import check_requirements
    from opinel.utils.profiles import AWSProfiles
except Exception as e:
    print('Error: Scout2 depends on the opinel package. Install all the requirements with the following command:')
    print('  $ pip install -r requirements.txt')
    print(e)
    sys.exit(42)

from opinel.utils.console import printDebug, printError, printException, printInfo
from opinel.utils.globals import manage_dictionary

from ScoutSuite.providers.aws.configs.services import AWSServicesConfig
from ScoutSuite.providers.base.configs.browser import combine_paths, get_object_at, get_value_at
from ScoutSuite.providers.aws.services.vpc import put_cidr_name
from ScoutSuite.providers.base.provider import BaseProvider
from ScoutSuite.utils import ec2_classic


class AWSProvider(BaseProvider):
    """
    Implements provider for AWS
    """

    def __init__(self, profile=None, report_dir=None, timestamp=None, services=None, skipped_services=None, thread_config=4, **kwargs):
        services = [] if services is None else services
        skipped_services = [] if skipped_services is None else skipped_services

        self.metadata_path = '%s/metadata.json' % os.path.split(os.path.abspath(__file__))[0]

        self.sg_map = {}
        self.subnet_map = {}

        self.profile = profile[0] if profile else 'default'
        self.aws_account_id = None
        self.services_config = AWSServicesConfig

        self.provider_code = 'aws'
        self.provider_name = 'Amazon Web Services'

        super(AWSProvider, self).__init__(report_dir, timestamp, services, skipped_services, thread_config)

    def authenticate(self, profile, csv_credentials, mfa_serial, mfa_code, **kwargs):
        """
        Implement authentication for the AWS provider
        :return:
        """
        self.credentials = read_creds(profile[0], csv_credentials, mfa_serial, mfa_code)
        self.aws_account_id = get_aws_account_id(self.credentials)

        if self.credentials['AccessKeyId'] is None:
            return False
        else:
            return True

    def preprocessing(self, ip_ranges=None, ip_ranges_name_key=None):
        """
        Tweak the AWS config to match cross-service resources and clean any fetching artifacts

        :param ip_ranges:
        :param ip_ranges_name_key:
        :return: None
        """
        ip_ranges = [] if ip_ranges is None else ip_ranges
        self._map_all_sgs()
        self._map_all_subnets()
        self._set_emr_vpc_ids()
        # self.parse_elb_policies()

        # Various data processing calls
        self._check_ec2_zone_distribution()
        self._add_security_group_name_to_ec2_grants()
        self._add_last_snapshot_date_to_ec2_volumes()
        self._process_cloudtrail_trails(self.services['cloudtrail'])
        self._add_cidr_display_name(ip_ranges, ip_ranges_name_key)
        self._merge_route53_and_route53domains()
        self._match_instances_and_roles()
        self._match_iam_policies_and_buckets()

        super(AWSProvider, self).preprocessing()

    def postprocessing(self, current_time, ruleset):
        """

        :param current_time:
        :param ruleset:
        :return: None
        """
        super(AWSProvider, self).postprocessing(current_time, ruleset)

    def _add_cidr_display_name(self, ip_ranges, ip_ranges_name_key):
        if len(ip_ranges):
            callback_args = {'ip_ranges': ip_ranges, 'ip_ranges_name_key': ip_ranges_name_key}
            self._go_to_and_do(self.services['ec2'],
                               ['regions', 'vpcs', 'security_groups', 'rules', 'protocols', 'ports'],
                               ['services', 'ec2'],
                               put_cidr_name,
                               callback_args)

    def _add_security_group_name_to_ec2_grants(self):
        """
        Github issue #24: display the security group names in the list of grants (added here to have ligher JS code)
        """
        self._go_to_and_do(self.services['ec2'],
                           ['regions', 'vpcs', 'security_groups', 'rules', 'protocols', 'ports', 'security_groups'],
                           [],
                           self.add_security_group_name_to_ec2_grants_callback,
                           {'AWSAccountId': self.aws_account_id})

    def _check_ec2_zone_distribution(self):
        regions = self.services['ec2']['regions'].values()
        self.services['ec2']['number_of_regions_with_instances'] = sum(r['instances_count'] > 0 for r in regions)

    def _add_last_snapshot_date_to_ec2_volumes(self):
        for region in self.services['ec2']['regions'].values():
            for volumeId, volume in region.get('volumes').items():
                completed_snapshots = [s for s in region['snapshots'].values() if s['VolumeId'] == volumeId and s['State'] == 'completed']
                sorted_snapshots = sorted(completed_snapshots, key=lambda s: s['StartTime'], reverse=True)
                volume['LastSnapshotDate'] = sorted_snapshots[0]['StartTime'] if len(sorted_snapshots) > 0 else None

    def add_security_group_name_to_ec2_grants_callback(self, current_config, path, current_path, ec2_grant, callback_args):
        sg_id = ec2_grant['GroupId']
        if sg_id in current_path:
            target = current_path[:(current_path.index(sg_id) + 1)]
            ec2_grant['GroupName'] = get_value_at(self.services['ec2'], target, 'name')
        elif ec2_grant['UserId'] == callback_args['AWSAccountId']:
            if 'VpcId' in ec2_grant:
                target = current_path[:(current_path.index('vpcs') + 1)]
                target.append(ec2_grant['VpcId'])
                target.append('security_groups')
                target.append(sg_id)
            else:
                target = current_path[:(current_path.index('security_groups') + 1)]
                target.append(sg_id)
            ec2_grant['GroupName'] = get_value_at(self.services['ec2'], target, 'name')

    def _process_cloudtrail_trails(self, cloudtrail_config):
        printInfo('Processing CloudTrail config...')
        global_events_logging = []
        data_logging_trails_count = 0
        for region in cloudtrail_config['regions']:
            for trail_id in cloudtrail_config['regions'][region]['trails']:
                trail = cloudtrail_config['regions'][region]['trails'][trail_id]
                if 'HomeRegion' in trail and trail['HomeRegion'] != region:
                    # Part of a multi-region trail, skip until we find the whole object
                    continue
                if trail['IncludeGlobalServiceEvents'] and trail['IsLogging']:
                    global_events_logging.append((region, trail_id,))
                # Any wildcard logging?
                if trail.get('wildcard_data_logging', False):
                    data_logging_trails_count += 1

        cloudtrail_config['data_logging_trails_count'] = data_logging_trails_count
        cloudtrail_config['IncludeGlobalServiceEvents'] = len(global_events_logging) > 0
        cloudtrail_config['DuplicatedGlobalServiceEvents'] = len(global_events_logging) > 1

    def process_network_acls_callback(self, current_config, path, current_path, privateip_id, callback_args):
        # Check if the network ACL allows all traffic from all IP addresses
        self._process_network_acls_check_for_allow_all(current_config, 'ingress')
        self._process_network_acls_check_for_allow_all(current_config, 'egress')
        # Check if the network ACL only has the default rules
        self._process_network_acls_check_for_aws_default(current_config, 'ingress')
        self._process_network_acls_check_for_aws_default(current_config, 'egress')

    def _process_network_acls_check_for_allow_all(self, network_acl, direction):
        network_acl['allow_all_%s_traffic' % direction] = 0
        for rule_number in network_acl['rules'][direction]:
            rule = network_acl['rules'][direction][rule_number]
            if rule['RuleAction'] == 'deny':
                # If a deny rule appears before an allow all, do not raise the flag
                break
            if (rule['CidrBlock'] == '0.0.0.0/0') and (rule['RuleAction'] == 'allow') and (
                    rule['port_range'] == '1-65535') and (rule['protocol'] == 'ALL'):
                network_acl['allow_all_%s_traffic' % direction] = rule_number
                break

    def _process_network_acls_check_for_aws_default(self, network_acl, direction):
        if len(network_acl['rules'][direction]) == 2 and int(
                network_acl['allow_all_%s_traffic' % direction]) > 0 and '100' in network_acl['rules'][direction]:
            # Assume it is AWS' default rules because there are 2 rules (100 and 65535) and the first rule allows all traffic
            network_acl['use_default_%s_rules' % direction] = True
        else:
            network_acl['use_default_%s_rules' % direction] = False

    def list_ec2_network_attack_surface_callback(self, current_config, path, current_path, privateip_id, callback_args):
        manage_dictionary(self.services['ec2'], 'external_attack_surface', {})
        if 'Association' in current_config and current_config['Association']:
            public_ip = current_config['Association']['PublicIp']
            self._security_group_to_attack_surface(self.services['ec2']['external_attack_surface'],
                                                   public_ip, current_path,
                                                   [g['GroupId'] for g in current_config['Groups']],
                                                   [])
        # IPv6
        if 'Ipv6Addresses' in current_config and len(current_config['Ipv6Addresses']) > 0:
            for ipv6 in current_config['Ipv6Addresses']:
                ip = ipv6['Ipv6Address']
                self._security_group_to_attack_surface(self.services['ec2']['external_attack_surface'],
                                                       ip, current_path,
                                                       [g['GroupId'] for g in current_config['Groups']], [])

    def _map_all_sgs(self):
        sg_map = dict()
        self._go_to_and_do(self.services['ec2'],
                           ['regions', 'vpcs', 'security_groups'],
                           ['services', 'ec2'],
                           self.map_resource,
                           {'map': sg_map})
        self.sg_map = sg_map

    def _map_all_subnets(self):
        subnet_map = dict()
        self._go_to_and_do(self.services['vpc'],
                           ['regions', 'vpcs', 'subnets'],
                           ['services', 'vpc'],
                           self.map_resource,
                           {'map': subnet_map})
        self.subnet_map = subnet_map

    def map_resource(self, current_config, path, current_path, resource_id, callback_args):
        if resource_id not in callback_args['map']:
            callback_args['map'][resource_id] = {'region': current_path[3]}
            if len(current_path) > 5:
                callback_args['map'][resource_id]['vpc_id'] = current_path[5]

    def _match_iam_policies_and_buckets(self):
        s3_info = self.services['s3']
        iam_info = self.services['iam']
        if 'Action' in iam_info['permissions']:
            for action in (x for x in iam_info['permissions']['Action'] if
                           ((x.startswith('s3:') and x != 's3:ListAllMyBuckets') or (x == '*'))):
                for iam_entity in iam_info['permissions']['Action'][action]:
                    if 'Allow' in iam_info['permissions']['Action'][action][iam_entity]:
                        for allowed_iam_entity in iam_info['permissions']['Action'][action][iam_entity]['Allow']:
                            # For resource statements, we can easily rely on the existing permissions structure
                            if 'Resource' in iam_info['permissions']['Action'][action][iam_entity]['Allow'][
                                allowed_iam_entity]:
                                for full_path in (x for x in
                                                  iam_info['permissions']['Action'][action][iam_entity]['Allow'][
                                                      allowed_iam_entity]['Resource'] if
                                                  x.startswith('arn:aws:s3:') or x == '*'):
                                    parts = full_path.split('/')
                                    bucket_name = parts[0].split(':')[-1]
                                    self._update_iam_permissions(s3_info, bucket_name, iam_entity, allowed_iam_entity,
                                                                iam_info['permissions']['Action'][action][iam_entity][
                                                                    'Allow'][allowed_iam_entity]['Resource'][full_path])
                            # For notresource statements, we must fetch the policy document to determine which buckets are not protected
                            if 'NotResource' in iam_info['permissions']['Action'][action][iam_entity]['Allow'][
                                allowed_iam_entity]:
                                for full_path in (x for x in
                                                  iam_info['permissions']['Action'][action][iam_entity]['Allow'][
                                                      allowed_iam_entity]['NotResource'] if
                                                  x.startswith('arn:aws:s3:') or x == '*'):
                                    for policy_type in ['InlinePolicies', 'ManagedPolicies']:
                                        if policy_type in \
                                                iam_info['permissions']['Action'][action][iam_entity]['Allow'][
                                                    allowed_iam_entity]['NotResource'][full_path]:
                                            for policy in \
                                                    iam_info['permissions']['Action'][action][iam_entity]['Allow'][
                                                        allowed_iam_entity]['NotResource'][full_path][policy_type]:
                                                self._update_bucket_permissions(s3_info, iam_info, action, iam_entity,
                                                                                allowed_iam_entity, full_path,
                                                                                policy_type,
                                                                                policy)

    def _update_bucket_permissions(self, s3_info, iam_info, action, iam_entity, allowed_iam_entity, full_path,
                                   policy_type,
                                   policy_name):
        allowed_buckets = []
        # By default, all buckets are allowed
        for bucket_name in s3_info['buckets']:
            allowed_buckets.append(bucket_name)
        if policy_type == 'InlinePolicies':
            policy = iam_info[iam_entity.title()][allowed_iam_entity]['Policies'][policy_name]['PolicyDocument']
        elif policy_type == 'ManagedPolicies':
            policy = iam_info['ManagedPolicies'][policy_name]['PolicyDocument']
        else:
            printError('Error, found unknown policy type.')
        for statement in policy['Statement']:
            for target_path in statement['NotResource']:
                parts = target_path.split('/')
                bucket_name = parts[0].split(':')[-1]
                path = '/' + '/'.join(parts[1:]) if len(parts) > 1 else '/'
                if (path == '/' or path == '/*') and (bucket_name in allowed_buckets):
                    # Remove bucket from list
                    allowed_buckets.remove(bucket_name)
                elif bucket_name == '*':
                    allowed_buckets = []
        policy_info = {}
        policy_info[policy_type] = {}
        policy_info[policy_type][policy_name] = \
            iam_info['permissions']['Action'][action][iam_entity]['Allow'][allowed_iam_entity]['NotResource'][
                full_path][
                policy_type][policy_name]
        for bucket_name in allowed_buckets:
            self._update_iam_permissions(s3_info, bucket_name, iam_entity, allowed_iam_entity, policy_info)

    def _update_iam_permissions(self, s3_info, bucket_name, iam_entity, allowed_iam_entity, policy_info):
        if bucket_name != '*' and bucket_name in s3_info['buckets']:
            bucket = s3_info['buckets'][bucket_name]
            manage_dictionary(bucket, iam_entity, {})
            manage_dictionary(bucket, iam_entity + '_count', 0)
            if not allowed_iam_entity in bucket[iam_entity]:
                bucket[iam_entity][allowed_iam_entity] = {}
                bucket[iam_entity + '_count'] = bucket[iam_entity + '_count'] + 1

            if 'inline_policies' in policy_info:
                manage_dictionary(bucket[iam_entity][allowed_iam_entity], 'inline_policies', {})
                bucket[iam_entity][allowed_iam_entity]['inline_policies'].update(policy_info['inline_policies'])
            if 'policies' in policy_info:
                manage_dictionary(bucket[iam_entity][allowed_iam_entity], 'policies', {})
                bucket[iam_entity][allowed_iam_entity]['policies'].update(policy_info['policies'])
        elif bucket_name == '*':
            for bucket in s3_info['buckets']:
                self._update_iam_permissions(s3_info, bucket, iam_entity, allowed_iam_entity, policy_info)
            pass
        else:
            # Could be an error or cross-account access, ignore...
            pass

    def match_network_acls_and_subnets_callback(self, current_config, path, current_path, acl_id, callback_args):
        for association in current_config['Associations']:
            subnet_path = current_path[:-1] + ['subnets', association['SubnetId']]
            subnet = get_object_at(self, subnet_path)
            subnet['network_acl'] = acl_id

    def match_instances_and_subnets_callback(self, current_config, path, current_path, instance_id, callback_args):
        subnet_id = current_config['SubnetId']
        if subnet_id:
            vpc = self.subnet_map[subnet_id]
            subnet = self.services['vpc']['regions'][vpc['region']]['vpcs'][vpc['vpc_id']]['subnets'][subnet_id]
            manage_dictionary(subnet, 'instances', [])
            if instance_id not in subnet['instances']:
                subnet['instances'].append(instance_id)

    def _match_instances_and_roles(self):
        printInfo('Matching EC2 instances and IAM roles...')
        ec2_config = self.services['ec2']
        iam_config = self.services['iam']
        role_instances = {}
        for r in ec2_config['regions']:
            for v in ec2_config['regions'][r]['vpcs']:
                if 'instances' in ec2_config['regions'][r]['vpcs'][v]:
                    for i in ec2_config['regions'][r]['vpcs'][v]['instances']:
                        instance_profile = ec2_config['regions'][r]['vpcs'][v]['instances'][i]['IamInstanceProfile']
                        instance_profile_id = instance_profile['Id'] if instance_profile else None
                        if instance_profile_id:
                            manage_dictionary(role_instances, instance_profile_id, [])
                            role_instances[instance_profile_id].append(i)
        for role_id in iam_config['roles']:
            iam_config['roles'][role_id]['instances_count'] = 0
            for instance_profile_id in iam_config['roles'][role_id]['instance_profiles']:
                if instance_profile_id in role_instances:
                    iam_config['roles'][role_id]['instance_profiles'][instance_profile_id]['instances'] = \
                        role_instances[instance_profile_id]
                    iam_config['roles'][role_id]['instances_count'] += len(role_instances[instance_profile_id])

    def match_roles_and_cloudformation_stacks_callback(self, current_config, path, current_path, stack_id, callback_args):
        if 'RoleARN' not in current_config:
            return
        role_arn = current_config.pop('RoleARN')
        current_config['iam_role'] = self._get_role_info('arn', role_arn)

    def match_roles_and_vpc_flowlogs_callback(self, current_config, path, current_path, flowlog_id, callback_args):
        if 'DeliverLogsPermissionArn' not in current_config:
            return
        delivery_role_arn = current_config.pop('DeliverLogsPermissionArn')
        current_config['delivery_role'] = self._get_role_info('arn', delivery_role_arn)

    def _get_role_info(self, attribute_name, attribute_value):
        iam_role_info = {'name': None, 'id': None}
        for role_id in self.services['iam']['roles']:
            if self.services['iam']['roles'][role_id][attribute_name] == attribute_value:
                iam_role_info['name'] = self.services['iam']['roles'][role_id]['name']
                iam_role_info['id'] = role_id
                break
        return iam_role_info

    def process_vpc_peering_connections_callback(self, current_config, path, current_path, pc_id, callback_args):

        # Create a list of peering connection IDs in each VPC
        info = 'AccepterVpcInfo' if current_config['AccepterVpcInfo']['OwnerId'] == self.aws_account_id else 'RequesterVpcInfo'
        region = current_path[current_path.index('regions') + 1]
        vpc_id = current_config[info]['VpcId']
        if vpc_id not in self.services['vpc']['regions'][region]['vpcs']:
            region = current_config['AccepterVpcInfo']['Region']
            target = self.services['vpc']['regions'][region]['vpcs'][vpc_id]
        else:
            target = self.services['vpc']['regions'][region]['vpcs'][vpc_id]
        manage_dictionary(target, 'peering_connections', [])
        if pc_id not in target['peering_connections']:
            target['peering_connections'].append(pc_id)

        # VPC information for the peer'd VPC
        current_config['peer_info'] = copy.deepcopy(
            current_config['AccepterVpcInfo' if info == 'RequesterVpcInfo' else 'RequesterVpcInfo'])
        if 'PeeringOptions' in current_config['peer_info']:
            current_config['peer_info'].pop('PeeringOptions')
        if hasattr(self, 'organization') and current_config['peer_info']['OwnerId'] in self.organization:
            current_config['peer_info']['name'] = self.organization[current_config['peer_info']['OwnerId']][
                'Name']
        else:
            current_config['peer_info']['name'] = current_config['peer_info']['OwnerId']

    def match_security_groups_and_resources_callback(self, current_config, path, current_path, resource_id, callback_args):
        service = current_path[1]
        original_resource_path = combine_paths(copy.deepcopy(current_path), [resource_id])
        resource = get_object_at(self, original_resource_path)
        if not 'resource_id_path' in callback_args:
            resource_type = current_path[-1]
            resource_path = copy.deepcopy(current_path)
            resource_path.append(resource_id)
        else:
            resource_path = combine_paths(copy.deepcopy(current_path), callback_args['resource_id_path'])
            resource_id = resource_path[-1]
            resource_type = resource_path[-2]
        if 'status_path' in callback_args:
            status_path = combine_paths(copy.deepcopy(original_resource_path), callback_args['status_path'])
            resource_status = get_object_at(self, status_path).replace('.', '_')
        else:
            resource_status = None
        unknown_vpc_id = True if current_path[4] != 'vpcs' else False
        # Issue 89 & 91 : can instances have no security group?
        try:
            try:
                sg_attribute = get_object_at(resource, callback_args['sg_list_attribute_name'])
            except:
                return
            if type(sg_attribute) != list:
                sg_attribute = [sg_attribute]
            for resource_sg in sg_attribute:
                if type(resource_sg) == dict:
                    sg_id = resource_sg[callback_args['sg_id_attribute_name']]
                else:
                    sg_id = resource_sg
                if unknown_vpc_id:
                    vpc_id = self.sg_map[sg_id]['vpc_id']
                    sg_base_path = copy.deepcopy(current_path[0:4])
                    sg_base_path[1] = 'ec2'
                    sg_base_path = sg_base_path + ['vpcs', vpc_id, 'security_groups']
                else:
                    sg_base_path = copy.deepcopy(current_path[0:6])
                    sg_base_path[1] = 'ec2'
                    sg_base_path.append('security_groups')
                sg_path = copy.deepcopy(sg_base_path)
                sg_path.append(sg_id)
                sg = get_object_at(self, sg_path)
                # Add usage information
                manage_dictionary(sg, 'used_by', {})
                manage_dictionary(sg['used_by'], service, {})
                manage_dictionary(sg['used_by'][service], 'resource_type', {})
                manage_dictionary(sg['used_by'][service]['resource_type'], resource_type, {} if resource_status else [])
                if resource_status:
                    manage_dictionary(sg['used_by'][service]['resource_type'][resource_type], resource_status, [])
                    if not resource_id in sg['used_by'][service]['resource_type'][resource_type][resource_status]:
                        sg['used_by'][service]['resource_type'][resource_type][resource_status].append(resource_id)
                else:
                    sg['used_by'][service]['resource_type'][resource_type].append(resource_id)
        except Exception as e:
            region = current_path[3]
            vpc_id = current_path[5]
            if vpc_id == ec2_classic and resource_type == 'elbs':
                pass
            else:
                printError('Failed to parse %s in %s in %s' % (resource_type, vpc_id, region))
                printException(e)

    def _merge_route53_and_route53domains(self):
        if 'route53domains' not in self.services:
            return
        # TODO fix this
        self.services['route53'].update(self.services['route53domains'])
        self.services.pop('route53domains')

    def _set_emr_vpc_ids(self):
        clear_list = []
        self._go_to_and_do(self.services['emr'],
                           ['regions', 'vpcs'],
                           ['services', 'emr'],
                           self.set_emr_vpc_ids_callback,
                           {'clear_list': clear_list})
        for region in clear_list:
            self.services['emr']['regions']['region']['vpcs'].pop('TODO')

    def set_emr_vpc_ids_callback(self, current_config, path, current_path, vpc_id, callback_args):
        if vpc_id != 'TODO':
            return
        region = current_path[3]
        vpc_id = sg_id = subnet_id = None
        pop_list = []
        for cluster_id in current_config['clusters']:
            cluster = current_config['clusters'][cluster_id]
            if 'EmrManagedMasterSecurityGroup' in cluster['Ec2InstanceAttributes']:
                sg_id = cluster['Ec2InstanceAttributes']['EmrManagedMasterSecurityGroup']
            elif 'RequestedEc2SubnetIds' in cluster['Ec2InstanceAttributes']:
                subnet_id = cluster['Ec2InstanceAttributes']['RequestedEc2SubnetIds']
            else:
                printError('Unable to determine VPC id for EMR cluster %s' % str(cluster_id))
                continue
            if sg_id in self.sg_map:
                vpc_id = self.sg_map[sg_id]['vpc_id']
                pop_list.append(cluster_id)
            else:
                sid_found = False
                if subnet_id:
                    for sid in subnet_id:
                        if sid in self.subnet_map:
                            vpc_id = self.subnet_map[sid]['vpc_id']
                            pop_list.append(cluster_id)
                            sid_found = True
                if not sid_found:
                    printError('Unable to determine VPC id for %s' % (str(subnet_id) if subnet_id else str(sg_id)))
                    continue
            if vpc_id:
                region_vpcs_config = get_object_at(current_path)
                manage_dictionary(region_vpcs_config, vpc_id, {'clusters': {}})
                region_vpcs_config[vpc_id]['clusters'][cluster_id] = cluster
        for cluster_id in pop_list:
            current_config['clusters'].pop(cluster_id)
        if len(current_config['clusters']) == 0:
            callback_args['clear_list'].append(region)

    def _parse_elb_policies(self):
        if 'elb' in self.services:
            self._go_to_and_do(self.services['elb'],
                               ['regions'],
                               [],
                               self.parse_elb_policies_callback,
                               {})

        # if 'elbv2' in self.config['services']:
        # Do something too here...

    def parse_elb_policies_callback(self, current_config, path, current_path, region_id, callback_args):
        region_config = get_object_at(['services', 'elb', ] + current_path + [region_id])
        region_config['elb_policies'] = current_config['elb_policies']
        for policy_id in region_config['elb_policies']:
            if region_config['elb_policies'][policy_id]['PolicyTypeName'] != 'SSLNegotiationPolicyType':
                continue
            # protocols, options, ciphers
            policy = region_config['elb_policies'][policy_id]
            protocols = {}
            options = {}
            ciphers = {}
            for attribute in policy['PolicyAttributeDescriptions']:
                if attribute['AttributeName'] in ['Protocol-SSLv3', 'Protocol-TLSv1', 'Protocol-TLSv1.1',
                                                  'Protocol-TLSv1.2']:
                    protocols[attribute['AttributeName']] = attribute['AttributeValue']
                elif attribute['AttributeName'] in ['Server-Defined-Cipher-Order']:
                    options[attribute['AttributeName']] = attribute['AttributeValue']
                elif attribute['AttributeName'] == 'Reference-Security-Policy':
                    policy['reference_security_policy'] = attribute['AttributeValue']
                else:
                    ciphers[attribute['AttributeName']] = attribute['AttributeValue']
                policy['protocols'] = protocols
                policy['options'] = options
                policy['ciphers'] = ciphers
                # TODO: pop ?

    def sort_vpc_flow_logs_callback(self, current_config, path, current_path, flow_log_id, callback_args):
        attached_resource = current_config['ResourceId']
        if attached_resource.startswith('vpc-'):
            vpc_path = combine_paths(current_path[0:4], ['vpcs', attached_resource])
            try:
                attached_vpc = get_object_at(self, vpc_path)
            except Exception as e:
                printDebug(
                    'It appears that the flow log %s is attached to a resource that was previously deleted (%s).' % (
                        flow_log_id, attached_resource))
                return
            manage_dictionary(attached_vpc, 'flow_logs', [])
            if flow_log_id not in attached_vpc['flow_logs']:
                attached_vpc['flow_logs'].append(flow_log_id)
            for subnet_id in attached_vpc['subnets']:
                manage_dictionary(attached_vpc['subnets'][subnet_id], 'flow_logs', [])
                if flow_log_id not in attached_vpc['subnets'][subnet_id]['flow_logs']:
                    attached_vpc['subnets'][subnet_id]['flow_logs'].append(flow_log_id)
        elif attached_resource.startswith('subnet-'):
            subnet_path = combine_paths(current_path[0:4],
                                        ['vpcs', self.subnet_map[attached_resource]['vpc_id'], 'subnets', attached_resource])
            subnet = get_object_at(self, subnet_path)
            manage_dictionary(subnet, 'flow_logs', [])
            if flow_log_id not in subnet['flow_logs']:
                subnet['flow_logs'].append(flow_log_id)
            # TODO this is pre-merge (from Loic) code
            # all_vpcs = get_object_at(self, combine_paths(current_path[0:2], ['vpcs']))
            # for vpc in self.services['vpc']:
            #     if attached_resource in all_vpcs[vpc]['subnets']:
            #         manage_dictionary(all_vpcs[vpc]['subnets'][attached_resource], 'flow_logs', [])
            #         if flow_log_id not in all_vpcs[vpc]['subnets'][attached_resource]['flow_logs']:
            #             all_vpcs[vpc]['subnets'][attached_resource]['flow_logs'].append(flow_log_id)
            #         break
        else:
            printError('Resource %s attached to flow logs is not handled' % attached_resource)

    def get_db_attack_surface(self, current_config, path, current_path, db_id, callback_args):
        service = current_path[1]
        service_config = self.services[service]
        manage_dictionary(service_config, 'external_attack_surface', {})
        if (service == 'redshift' or service == 'rds') and 'PubliclyAccessible' in current_config and current_config[
            'PubliclyAccessible']:
            public_dns = current_config['Endpoint']['Address']
            listeners = [current_config['Endpoint']['Port']]
            security_groups = current_config['VpcSecurityGroups']
            self._security_group_to_attack_surface(service_config['external_attack_surface'], public_dns,
                                                   current_path, [g['VpcSecurityGroupId'] for g in security_groups],
                                                   listeners)
        elif 'ConfigurationEndpoint' in current_config:
            public_dns = current_config['ConfigurationEndpoint']['Address'].replace('.cfg',
                                                                                    '')  # TODO : get the proper addresss
            listeners = [current_config['ConfigurationEndpoint']['Port']]
            security_groups = current_config['SecurityGroups']
            self._security_group_to_attack_surface(service_config['external_attack_surface'], public_dns,
                                                   current_path, [g['SecurityGroupId'] for g in security_groups],
                                                   listeners)
            # TODO :: Get Redis endpoint information

    def get_lb_attack_surface(self, current_config, path, current_path, elb_id, callback_args):
        public_dns = current_config['DNSName']
        elb_config = self.services[current_path[1]]
        manage_dictionary(elb_config, 'external_attack_surface', {})
        if current_path[1] == 'elbv2' and current_config['Type'] == 'network':
            # Network LBs do not have a security group, lookup listeners instead
            manage_dictionary(elb_config['external_attack_surface'], public_dns, {'protocols': {}})
            for listener in current_config['listeners']:
                protocol = current_config['listeners'][listener]['Protocol']
                manage_dictionary(elb_config['external_attack_surface'][public_dns]['protocols'], protocol,
                                  {'ports': {}})
                manage_dictionary(elb_config['external_attack_surface'][public_dns]['protocols'][protocol]['ports'],
                                  listener, {'cidrs': []})
                elb_config['external_attack_surface'][public_dns]['protocols'][protocol]['ports'][listener][
                    'cidrs'].append({'CIDR': '0.0.0.0/0'})
        elif current_path[1] == 'elbv2' and current_config['Scheme'] == 'internet-facing':
            vpc_id = current_path[5]
            elb_config['external_attack_surface'][public_dns] = {'protocols': {}}
            security_groups = [g['GroupId'] for g in current_config['security_groups']]
            listeners = []
            for listener in current_config['listeners']:
                listeners.append(listener)
            self._security_group_to_attack_surface(elb_config['external_attack_surface'], public_dns,
                                                   current_path, security_groups, listeners)
        elif current_config['Scheme'] == 'internet-facing':
            # Classic ELbs do not have a security group, lookup listeners instead
            public_dns = current_config['DNSName']
            manage_dictionary(elb_config['external_attack_surface'], public_dns, {'protocols': {'TCP': {'ports': {}}}})
            for listener in current_config['listeners']:
                manage_dictionary(elb_config['external_attack_surface'][public_dns]['protocols']['TCP']['ports'],
                                  listener, {'cidrs': []})
                elb_config['external_attack_surface'][public_dns]['protocols']['TCP']['ports'][listener][
                    'cidrs'].append({'CIDR': '0.0.0.0/0'})

    def _security_group_to_attack_surface(self, attack_surface_config, public_ip, current_path,
                                          security_groups, listeners=None):
        listeners = [] if listeners is None else listeners
        manage_dictionary(attack_surface_config, public_ip, {'protocols': {}})
        for sg_id in security_groups:
            sg_path = copy.deepcopy(current_path[0:6])
            sg_path[1] = 'ec2'
            sg_path.append('security_groups')
            sg_path.append(sg_id)
            sg_path.append('rules')
            sg_path.append('ingress')
            ingress_rules = get_object_at(self, sg_path)
            public_ip_grants = {}
            for p in ingress_rules['protocols']:
                for port in ingress_rules['protocols'][p]['ports']:
                    if len(listeners) == 0 and 'cidrs' in ingress_rules['protocols'][p]['ports'][port]:
                        manage_dictionary(attack_surface_config[public_ip]['protocols'], p, {'ports': {}})
                        manage_dictionary(attack_surface_config[public_ip]['protocols'][p]['ports'], port,
                                          {'cidrs': []})
                        attack_surface_config[public_ip]['protocols'][p]['ports'][port]['cidrs'] += \
                            ingress_rules['protocols'][p]['ports'][port]['cidrs']
                    else:
                        ports = port.split('-')
                        if len(ports) > 1:
                            port_min = int(ports[0])
                            port_max = int(ports[1])
                        elif port == 'N/A':
                            port_min = port_max = None
                        elif port == 'ALL':
                            port_min = 0
                            port_max = 65535
                        else:
                            port_min = port_max = int(port)
                        for listener in listeners:
                            if (port_min and port_max) and \
                                    int(listener) > port_min and int(listener) < port_max and \
                                    'cidrs' in ingress_rules['protocols'][p]['ports'][port]:
                                manage_dictionary(attack_surface_config[public_ip]['protocols'], p, {'ports': {}})
                                manage_dictionary(attack_surface_config[public_ip]['protocols'][p]['ports'],
                                                  str(listener), {'cidrs': []})
                                attack_surface_config[public_ip]['protocols'][p]['ports'][str(listener)]['cidrs'] += \
                                    ingress_rules['protocols'][p]['ports'][port]['cidrs']



# -*- coding: utf-8 -*-

from ScoutSuite.providers.azure.configs.base import AzureBaseConfig


class KeyVaultConfig(AzureBaseConfig):
    targets = (
        ('vaults', 'Key Vaults', 'list_by_subscription', {}, False),
    )

    def __init__(self, thread_config):

        self.vaults = {}
        self.vaults_count = 0

        super(KeyVaultConfig, self).__init__(thread_config)

    def parse_vaults(self, vault, params):
        vault_dict = {}
        vault_dict['id'] = self.get_non_provider_id(vault.id)
        vault_dict['name'] = vault.name
        vault_dict['public_access_allowed'] = self._is_public_access_allowed(vault)

        self.vaults[vault_dict['id']] = vault_dict

    def _is_public_access_allowed(self, vault):
        return vault.properties.network_acls is None



# -*- coding: utf-8 -*-
"""
ELB-related classes and functions
"""
from opinel.utils.globals import manage_dictionary

from ScoutSuite.providers.aws.configs.regions import RegionalServiceConfig, RegionConfig, api_clients
from ScoutSuite.providers.aws.configs.vpc import VPCConfig
from ScoutSuite.utils import ec2_classic, get_keys



########################################
# ELBRegionConfig
########################################

class ELBRegionConfig(RegionConfig):
    """
    ELB configuration for a single AWS region
    """

    def parse_elb(self, global_params, region, lb):
        """

        :param global_params:
        :param region:
        :param elb:
        :return:
        """
        elb = {}
        elb['name'] = lb.pop('LoadBalancerName')
        vpc_id = lb['VPCId'] if 'VPCId' in lb and lb['VPCId'] else ec2_classic
        manage_dictionary(self.vpcs, vpc_id, VPCConfig(self.vpc_resource_types))
        get_keys(lb, elb, ['DNSName', 'CreatedTime', 'AvailabilityZones', 'Subnets', 'Scheme'])
        elb['security_groups'] = []
        for sg in lb['SecurityGroups']:
            elb['security_groups'].append({'GroupId': sg})
        manage_dictionary(elb, 'listeners', {})
        policy_names = []
        for l in lb['ListenerDescriptions']:
            listener = l['Listener']
            manage_dictionary(listener, 'policies', [])
            for policy_name in l['PolicyNames']:
                policy_id = self.get_non_provider_id(policy_name)
                listener['policies'].append(policy_id)
                if policy_id not in self.elb_policies:
                    policy_names.append(policy_name)
            elb['listeners'][l['Listener']['LoadBalancerPort']] = listener
        # Fetch LB policies here. This is not ideal, but the alternative is to download all policies and clean up after...
        if len(policy_names):
            policies = api_clients[region].describe_load_balancer_policies(LoadBalancerName = elb['name'], PolicyNames = policy_names)['PolicyDescriptions']
            for policy in policies:
                policy['name'] = policy.pop('PolicyName')
                policy_id = self.get_non_provider_id(policy['name'])
                self.elb_policies[policy_id] = policy
        manage_dictionary(elb, 'instances', [])
        for i in lb['Instances']:
            elb['instances'].append(i['InstanceId'])
        # Get attributes
        elb['attributes'] = api_clients[region].describe_load_balancer_attributes(LoadBalancerName=elb['name'])['LoadBalancerAttributes']
        self.vpcs[vpc_id].elbs[self.get_non_provider_id(elb['name'])] = elb



########################################
# ELBConfig
########################################

class ELBConfig(RegionalServiceConfig):
    """
    ELB configuration for all AWS regions
    """

    region_config_class = ELBRegionConfig

    def __init__(self, service_metadata, thread_config = 4):
        super(ELBConfig, self).__init__(service_metadata, thread_config)



# -*- coding: utf-8 -*-

# Python2 vs Python3
try:
    from Queue import Queue
except ImportError:
    from queue import Queue

from ScoutSuite.providers.base.configs.base import BaseConfig

from opinel.utils.aws import handle_truncated_response


class AWSBaseConfig(BaseConfig):

    def _is_provider(self, provider_name):
        return provider_name == 'aws'

    def _get_method(self, api_client, target_type, list_method_name):
        """
        Gets the appropriate method, required as each provider may have particularities

        :return:
        """

        method = getattr(api_client, list_method_name)

        return method

    def _get_targets(self, response_attribute, api_client, method, list_params, ignore_list_error):
        """
        Fetch the targets, required as each provider may have particularities

        :return:
        """

        if type(list_params) != list:
            list_params = [list_params]

        targets = []
        for lp in list_params:
            targets += handle_truncated_response(method, lp, [response_attribute])[response_attribute]

        return targets



# -*- coding: utf-8 -*-

from ScoutSuite.providers.azure.configs.base import AzureBaseConfig


class NetworkConfig(AzureBaseConfig):
    targets = (
        ('network_watchers', 'Network Watchers', 'list_all', {}, False),
        ('network_security_groups', 'Network Security Group', 'list_all', {}, False),
    )

    def __init__(self, thread_config):
        self.network_watchers = {}
        self.network_watchers_count = 0

        self.network_security_groups = {}
        self.network_security_groups_count = 0

        super(NetworkConfig, self).__init__(thread_config)

    def parse_network_watchers(self, network_watcher, params):
        network_watcher_dict = {}
        network_watcher_dict['id'] = network_watcher.id
        network_watcher_dict['name'] = network_watcher.name
        network_watcher_dict['provisioning_state'] = network_watcher.provisioning_state
        network_watcher_dict['location'] = network_watcher.location
        network_watcher_dict['etag'] = network_watcher.etag

        self.network_watchers[network_watcher_dict['id']] = network_watcher_dict

    def parse_network_security_groups(self, network_security_group, params):
        network_security_group_dict = {}
        network_security_group_dict['id'] = network_security_group.id
        network_security_group_dict['name'] = network_security_group.name
        network_security_group_dict['provisioning_state'] = network_security_group.provisioning_state
        network_security_group_dict['location'] = network_security_group.location
        network_security_group_dict['resource_guid'] = network_security_group.resource_guid
        network_security_group_dict['etag'] = network_security_group.etag

        network_security_group_dict['security_rules'] = self._parse_security_rules(network_security_group)

        exposed_ports = self._parse_exposed_ports(network_security_group)
        network_security_group_dict['exposed_ports'] = exposed_ports
        network_security_group_dict['exposed_port_ranges'] = self._format_ports(exposed_ports)

        self.network_security_groups[network_security_group_dict['id']] = network_security_group_dict

    def _parse_security_rules(self, network_security_group):
        security_rules = {}
        for sr in network_security_group.security_rules:
            security_rule_dict = {}
            security_rule_dict['id'] = sr.id
            security_rule_dict['name'] = sr.name
            security_rule_dict['allow'] = sr.access == "Allow"
            security_rule_dict['priority'] = sr.priority
            security_rule_dict['description'] = sr.description
            security_rule_dict['provisioning_state'] = sr.provisioning_state

            security_rule_dict['protocol'] = sr.protocol
            security_rule_dict['direction'] = sr.direction

            source_address_prefixes = self._merge_prefixes_or_ports(sr.source_address_prefix,
                                                                    sr.source_address_prefixes)
            security_rule_dict['source_address_prefixes'] = source_address_prefixes

            source_port_ranges = self._merge_prefixes_or_ports(sr.source_port_range, sr.source_port_ranges)
            security_rule_dict['source_port_ranges'] = source_port_ranges
            security_rule_dict['source_ports'] = self._parse_ports(source_port_ranges)

            destination_address_prefixes = self._merge_prefixes_or_ports(sr.destination_address_prefix,
                                                                         sr.destination_address_prefixes)
            security_rule_dict['destination_address_prefixes'] = destination_address_prefixes

            destination_port_ranges = self._merge_prefixes_or_ports(sr.destination_port_range,
                                                                    sr.destination_port_ranges)
            security_rule_dict['destination_port_ranges'] = destination_port_ranges
            security_rule_dict['destination_ports'] = self._parse_ports(destination_port_ranges)

            security_rule_dict['etag'] = sr.etag

            security_rules[security_rule_dict['id']] = security_rule_dict

        return security_rules

    def _parse_ports(self, port_ranges):
        ports = set()
        for pr in port_ranges:
            if pr == "*":
                for p in range(0, 65535 + 1):
                    ports.add(p)
                break
            elif "-" in pr:
                lower, upper = pr.split("-")
                for p in range(int(lower), int(upper) + 1):
                    ports.add(p)
            else:
                ports.add(int(pr))
        ports = list(ports)
        ports.sort()
        return ports

    def _parse_exposed_ports(self, network_security_group):
        exposed_ports = set()

        # Sort by priority.
        rules = network_security_group.default_security_rules + network_security_group.security_rules
        rules.sort(key=lambda x: x.priority, reverse=True)

        for sr in rules:
            if sr.direction == "Inbound" and (sr.source_address_prefix == "*"
                                              or sr.source_address_prefix == "Internet"):
                port_ranges = self._merge_prefixes_or_ports(sr.destination_port_range,
                                                                     sr.destination_port_ranges)
                ports = self._parse_ports(port_ranges)
                if sr.access == "Allow":
                    for p in ports:
                        exposed_ports.add(p)
                else:
                    for p in ports:
                        exposed_ports.discard(p)
        exposed_ports = list(exposed_ports)
        exposed_ports.sort()
        return exposed_ports

    def _merge_prefixes_or_ports(self, port_range, port_ranges):
        port_ranges = port_ranges if port_ranges else []
        if port_range:
            port_ranges.append(port_range)
        return port_ranges

    def _format_ports(self, ports):
        port_ranges = []
        start = None
        for i in range(0, 65535 + 1):
            if i in ports:
                if not start:
                    start = i
            else:
                if start:
                    if i - 1 == start:
                        port_ranges.append(str(start))
                    else:
                        port_ranges.append(str(start) + "-" + str(i - 1))
                    start = None
        return port_ranges



# -*- coding: utf-8 -*-

import re

from opinel.utils.console import printException, printInfo

from azure.mgmt.storage import StorageManagementClient
from azure.mgmt.monitor import MonitorManagementClient
from azure.mgmt.sql import SqlManagementClient

from azure.mgmt.security import SecurityCenter
from azure.mgmt.keyvault import KeyVaultManagementClient
from azure.mgmt.network import NetworkManagementClient
from azure.mgmt.redis import RedisManagementClient
from azure.mgmt.web import WebSiteManagementClient


def azure_connect_service(service, credentials, region_name=None):
    try:
        if service == 'storageaccounts':
            return StorageManagementClient(credentials.credentials, credentials.subscription_id)
        elif service == 'monitor':
            return MonitorManagementClient(credentials.credentials, credentials.subscription_id)
        elif service == 'sqldatabase':
            return SqlManagementClient(credentials.credentials, credentials.subscription_id)
        elif service == 'keyvault':
            return KeyVaultManagementClient(credentials.credentials, credentials.subscription_id)
        elif service == 'appgateway':
            return NetworkManagementClient(credentials.credentials, credentials.subscription_id)
        elif service == 'network':
            return NetworkManagementClient(credentials.credentials, credentials.subscription_id)
        elif service == 'rediscache':
            return RedisManagementClient(credentials.credentials, credentials.subscription_id)
        elif service == 'securitycenter':
            return SecurityCenter(credentials.credentials, credentials.subscription_id, '')
        elif service == 'appservice':
            return WebSiteManagementClient(credentials.credentials, credentials.subscription_id)
        elif service == 'loadbalancer':
            return NetworkManagementClient(credentials.credentials, credentials.subscription_id)
        else:
            printException('Service %s not supported' % service)
            return None

    except Exception as e:
        printException(e)
        return None


def get_resource_group_name(id):
    return re.findall("/resourceGroups/(.*?)/", id)[0]



# -*- coding: utf-8 -*-
"""
Redshift-related classes and functions
"""

from opinel.utils.aws import handle_truncated_response
from opinel.utils.globals import manage_dictionary

from ScoutSuite.providers.aws.configs.regions import RegionalServiceConfig, RegionConfig, api_clients
from ScoutSuite.providers.aws.configs.vpc import VPCConfig
from ScoutSuite.utils import ec2_classic


########################################
# RedshiftRegionConfig
########################################

class RedshiftRegionConfig(RegionConfig):
    """
    Redshift configuration for a single AWS region
    """

    def parse_cluster(self, global_params, region, cluster):
        """
        Parse a single Redshift cluster

        :param global_params:           Parameters shared for all regions
        :param region:                  Name of the AWS region
        :param cluster:                 Cluster
        """
        vpc_id = cluster.pop('VpcId') if 'VpcId' in cluster else ec2_classic
        manage_dictionary(self.vpcs, vpc_id, VPCConfig(self.vpc_resource_types))
        name = cluster.pop('ClusterIdentifier')
        cluster['name'] = name
        self.vpcs[vpc_id].clusters[name] = cluster


    def parse_parameter_group(self, global_params, region, parameter_group):
        """
        Parse a single Redshift parameter group and fetch all of its parameters

        :param global_params:           Parameters shared for all regions
        :param region:                  Name of the AWS region
        :param parameter_group:         Parameter group
        """
        pg_name = parameter_group.pop('ParameterGroupName')
        pg_id = self.get_non_provider_id(pg_name) # Name could be used as only letters digits or hyphens
        parameter_group['name'] = pg_name
        parameter_group['parameters'] = {}
        api_client = api_clients[region]
        parameters = handle_truncated_response(api_client.describe_cluster_parameters, {'ParameterGroupName': pg_name}, ['Parameters'])['Parameters']
        for parameter in parameters:
            param = {}
            param['value'] = parameter['ParameterValue']
            param['source'] = parameter['Source']
            parameter_group['parameters'][parameter['ParameterName']] = param
        (self).parameter_groups[pg_id] = parameter_group


    def parse_security_group(self, global_params, region, security_group):
        """
        Parse a single Redsfhit security group

        :param global_params:           Parameters shared for all regions
        :param region:                  Name of the AWS region
        :param security)_group:         Security group
        """
        name = security_group.pop('ClusterSecurityGroupName')
        security_group['name'] = name
        self.security_groups['name'] = security_group



########################################
# RedshiftConfig
########################################

class RedshiftConfig(RegionalServiceConfig):
    """
    Redshift configuration for all AWS regions
    """

    region_config_class = RedshiftRegionConfig

    def __init__(self, service_metadata, thread_config = 4):
        super(RedshiftConfig, self).__init__(service_metadata, thread_config)



# -*- coding: utf-8 -*-

from opinel.utils.console import printError, printException, printDebug

from ScoutSuite.providers.base.configs.services import BaseServicesConfig
from ScoutSuite.providers.gcp.services.cloudstorage import CloudStorageConfig
from ScoutSuite.providers.gcp.services.cloudsql import CloudSQLConfig
from ScoutSuite.providers.gcp.services.iam import IAMConfig
from ScoutSuite.providers.gcp.services.stackdriverlogging import StackdriverLoggingConfig
from ScoutSuite.providers.gcp.services.stackdrivermonitoring import StackdriverMonitoringConfig
from ScoutSuite.providers.gcp.services.computeengine import ComputeEngineConfig
from ScoutSuite.providers.gcp.services.cloudresourcemanager import CloudResourceManager

try:
    from ScoutSuite.providers.gcp.services.kubernetesengine_private import KubernetesEngineConfig
except ImportError:
    pass


class GCPServicesConfig(BaseServicesConfig):

    def __init__(self, metadata=None, thread_config=4, projects=None, **kwargs):

        projects = [] if projects is None else projects

        self.cloudresourcemanager = CloudResourceManager(thread_config=thread_config)
        self.cloudstorage = CloudStorageConfig(thread_config=thread_config)
        self.cloudsql = CloudSQLConfig(thread_config=thread_config)
        self.computeengine = ComputeEngineConfig(thread_config=thread_config)
        self.iam = IAMConfig(thread_config=thread_config)

        try:
            self.kubernetesengine = KubernetesEngineConfig(thread_config=thread_config)
        except NameError as e:
            pass

        self.stackdriverlogging = StackdriverLoggingConfig(thread_config=thread_config)
        # self.stackdrivermonitoring = StackdriverMonitoringConfig(thread_config=thread_config)

    def _is_provider(self, provider_name):
        return provider_name == 'gcp'

    def set_projects(self, projects):
        """
        Set the projects attribute of each of the configs. This is because before authentication (when configs
        are instanciated, the projects within an organization are not known).

        :param projects: List of projects to set
        :return: None
        """

        for c in vars(self):
            setattr(getattr(self, c), 'projects', projects)




# -*- coding: utf-8 -*-

import datetime

from ScoutSuite.providers.azure.configs.base import AzureBaseConfig


class MonitorConfig(AzureBaseConfig):
    time_format = "%Y-%m-%dT%H:%M:%S.%f"
    utc_now = datetime.datetime.utcnow()
    end_time = utc_now.strftime(time_format)
    timespan = datetime.timedelta(90)  # 90 days of timespan
    start_time = (utc_now - timespan).strftime(time_format)

    targets = (
        ('activity_logs', 'Activity Logs', 'list',
         {'filter': "eventTimestamp ge {} and eventTimestamp le {}".format(start_time, end_time)},
         False),
    )

    def __init__(self, thread_config):

        self.activity_logs = {}
        self.activity_logs['storage_accounts'] = {}

        super(MonitorConfig, self).__init__(thread_config)

    def parse_activity_logs(self, activity_log, params):
        if activity_log.resource_type.value == 'Microsoft.Storage/storageAccounts':
            self._parse_storage_account_logs(activity_log)

    def _parse_storage_account_logs(self, activity_log):
            storage_account_id = self.get_non_provider_id(activity_log.resource_id.lower())

            if storage_account_id not in self.activity_logs['storage_accounts']:
                self.activity_logs['storage_accounts'][storage_account_id] =\
                    {'access_keys_rotated': False}
                
            if activity_log.operation_name.value == 'Microsoft.Storage/storageAccounts/regenerateKey/action':
                self.activity_logs['storage_accounts'][storage_account_id]['access_keys_rotated'] = True



# -*- coding: utf-8 -*-
"""
EFS-related classes and functions
"""
from opinel.utils.aws import handle_truncated_response

from ScoutSuite.providers.aws.configs.regions import RegionalServiceConfig, RegionConfig, api_clients



########################################
# EFSRegionConfig
########################################

class EFSRegionConfig(RegionConfig):
    """
    EFS configuration for a single AWS region
    """

    def parse_file_system(self, global_params, region, file_system):
        """

        :param global_params:
        :param region:
        :param file_system:
        :return:
        """
        fs_id = file_system.pop('FileSystemId')
        if 'Name' in file_system:
            file_system['name'] = file_system.pop('Name')
        else:
            file_system['name'] = None
        # Get tags
        file_system['tags'] = handle_truncated_response(api_clients[region].describe_tags, {'FileSystemId': fs_id}, ['Tags'])['Tags']
        # Get mount targets
        mount_targets = handle_truncated_response(api_clients[region].describe_mount_targets, {'FileSystemId': fs_id}, ['MountTargets'])['MountTargets']
        file_system['mount_targets'] = {}
        for mt in mount_targets:
            mt_id = mt['MountTargetId']
            file_system['mount_targets'][mt_id] = mt
            # Get security groups
            file_system['mount_targets'][mt_id]['security_groups'] = api_clients[region].describe_mount_target_security_groups(MountTargetId = mt_id)['SecurityGroups']
        self.file_systems[fs_id] = file_system



########################################
# EFSConfig
########################################

class EFSConfig(RegionalServiceConfig):
    """
    EFS configuration for all AWS regions
    """

    region_config_class = EFSRegionConfig

    def __init__(self, service_metadata, thread_config = 4):
        super(EFSConfig, self).__init__(service_metadata, thread_config)



# -*- coding: utf-8 -*-

from ScoutSuite.providers.azure.configs.base import AzureBaseConfig
from ScoutSuite.providers.azure.utils import get_resource_group_name

class StorageAccountsConfig(AzureBaseConfig):
    targets = (
        ('storage_accounts', 'Storage Accounts', 'list', {}, False),
    )

    def __init__(self, thread_config):

        self.storage_accounts = {}
        self.storage_accounts_count = 0

        super(StorageAccountsConfig, self).__init__(thread_config)

    def parse_storage_accounts(self, storage_account, params):

        storage_account_dict = {}

        storage_account_dict['id'] = self.get_non_provider_id(storage_account.id.lower())
        storage_account_dict['name'] = storage_account.name
        storage_account_dict['https_traffic_enabled'] = storage_account.enable_https_traffic_only
        storage_account_dict['public_traffic_allowed'] = self._is_public_traffic_allowed(storage_account)
        storage_account_dict['blob_containers'] = self._parse_blob_containers(storage_account)

        self.storage_accounts[storage_account_dict['id']] = storage_account_dict

    def _is_public_traffic_allowed(self, storage_account):
        return storage_account.network_rule_set.default_action == "Allow"

    def _parse_blob_containers(self, storage_account):
        blob_containers = {}
        for container in storage_account.blob_containers:
            container_dict = {}
            container_dict['id'] = container.name
            container_dict['public_access_allowed'] = container.public_access != "None"
            blob_containers[container.name] = container_dict

        return blob_containers

    def _get_targets(self, response_attribute, api_client, method, list_params, ignore_list_error):
        if response_attribute == "Storage Accounts":
            return self._get_storage_accounts(api_client, method, list_params)
        else:
            return super(StorageAccountsConfig, self)._get_targets(response_attribute, api_client, method,
                                                                   list_params, ignore_list_error)

    def _get_storage_accounts(self, api_client, method, list_params):
        storage_accounts = []
        storage_accounts_raw = method(**list_params)
        for storage_account in storage_accounts_raw:
            resource_group_name = get_resource_group_name(storage_account.id)
            setattr(storage_account, "blob_containers", \
                    api_client.blob_containers.list(resource_group_name, storage_account.name).value)
            storage_accounts.append(storage_account)

        return storage_accounts



# -*- coding: utf-8 -*-

import json
import os
import re

from opinel.utils.fs import read_ip_ranges
from opinel.utils.console import printDebug, printError, printException

from ScoutSuite.utils import format_service_name


ip_ranges_from_args = 'ip-ranges-from-args'

re_aws_account_id = re.compile(r'_AWS_ACCOUNT_ID_')
re_ip_ranges_from_file = re.compile(r'_IP_RANGES_FROM_FILE_\((.*?)(,.*?)\)')
re_ip_ranges_from_local_file = re.compile(r'_IP_RANGES_FROM_LOCAL_FILE_\((.*?)(,.*?)\)')
re_strip_dots = re.compile(r'(_STRIPDOTS_\((.*?)\))')

testcases = [
    {
        'name': 'aws_account_id',
        'regex': re_aws_account_id
    },
    {
        'name': 'ip_ranges_from_file',
        'regex': re_ip_ranges_from_file
    },
    {
        'name': 'ip_ranges_from_local_file',
        'regex': re_ip_ranges_from_local_file
    }
]

class Rule(object):

    def to_string(self):
        return (str(vars(self)))
    
    def __init__(self, data_path, filename, rule_type, rule):
        self.data_path = data_path
        self.filename = filename
        self.rule_type = rule_type
        self.enabled = bool(self.get_attribute('enabled', rule, False))
        self.level = self.get_attribute('level', rule, '')
        self.args = self.get_attribute('args', rule, [])
        self.conditions = self.get_attribute('conditions', rule, [])
        self.key_suffix = self.get_attribute('key_suffix', rule, None)


    def get_attribute(self, name, rule, default_value):
        return rule[name] if name in list(rule.keys()) else default_value


    def set_definition(self, rule_definitions, attributes = None, ip_ranges = None, params = None):
        """
        Update every attribute of the rule by setting the argument values as necessary

        :param parameterized_input:
        :param arg_values:
        :param convert:
        :return:
        """
        attributes = [] if attributes is None else attributes
        ip_ranges = [] if ip_ranges is None else ip_ranges
        params = {} if params is None else params
        try:
            string_definition = rule_definitions[self.filename].string_definition
            # Load condition dependencies
            definition = json.loads(string_definition)
            definition['conditions'] += self.conditions
            loaded_conditions = []
            for condition in definition['conditions']:
                if condition[0].startswith('_INCLUDE_('):
                    include = re.findall(r'_INCLUDE_\((.*?)\)', condition[0])[0]
                    #new_conditions = load_data(include, key_name = 'conditions')
                    rules_path = '%s/%s' % (self.data_path, include)
                    with open(rules_path, 'rt') as f:
                        new_conditions = f.read()
                        for (i, value) in enumerate(condition[1]):
                            new_conditions = re.sub(condition[1][i], condition[2][i], new_conditions)
                        new_conditions = json.loads(new_conditions)['conditions']
                    loaded_conditions.append(new_conditions)
                else:
                    loaded_conditions.append(condition)
            definition['conditions'] = loaded_conditions
            string_definition = json.dumps(definition)
            # Set parameters
            parameters = re.findall(r'(_ARG_([a-zA-Z0-9]+)_)', string_definition)
            for param in parameters:
                index = int(param[1])
                if len(self.args) <= index:
                    string_definition = string_definition.replace(param[0], '')
                elif type(self.args[index]) == list:
                    value = '[ %s ]' % ', '.join('"%s"' % v for v in self.args[index])
                    string_definition = string_definition.replace('"%s"' % param[0], value)
                else:
                    string_definition = string_definition.replace(param[0], self.args[index])
            # Strip dots if necessary
            stripdots = re_strip_dots.findall(string_definition)
            for value in stripdots:
                string_definition = string_definition.replace(value[0], value[1].replace('.', ''))
            definition = json.loads(string_definition)
            # Set special values (IP ranges, AWS account ID, ...)
            for condition in definition['conditions']:
                if type(condition) != list or len(condition) == 1 or type(condition[2]) == list:
                    continue
                for testcase in testcases:
                    result = testcase['regex'].match(condition[2])
                    if result and (testcase['name'] == 'ip_ranges_from_file' or testcase['name'] == 'ip_ranges_from_local_file'):
                        filename = result.groups()[0]
                        conditions = result.groups()[1] if len(result.groups()) > 1 else []
                        # TODO :: handle comma here...
                        if filename == ip_ranges_from_args:
                            prefixes = []
                            for filename in ip_ranges:
                                prefixes += read_ip_ranges(filename, local_file = True, ip_only = True, conditions = conditions)
                            condition[2] = prefixes
                            break
                        else:
                            local_file = True if testcase['name'] == 'ip_ranges_from_local_file' else False
                            condition[2] = read_ip_ranges(filename, local_file = local_file, ip_only = True, conditions = conditions)
                            break
                        break
                    elif result:
                        condition[2] = params[testcase['name']]
                        break

            if len(attributes) == 0:
                attributes = [attr for attr in definition]
            for attr in attributes:
                if attr in definition:
                    setattr(self, attr, definition[attr])
            if hasattr(self, 'path'):
                self.service = format_service_name(self.path.split('.')[0])
            if not hasattr(self, 'key'):
                setattr(self, 'key', self.filename)
            setattr(self, 'key', self.key.replace('.json', ''))
            if self.key_suffix:
                setattr(self, 'key', '%s-%s' % (self.key, self.key_suffix))
        except Exception as e:
            # printException(e)
            printError('Failed to set definition %s: %s' % (self.filename, e))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import copy
import json
import os
import webbrowser

from opinel.utils.console import configPrintException, printInfo, printDebug
from opinel.utils.profiles import AWSProfiles

from ScoutSuite import AWSCONFIG
from ScoutSuite.output.html import Scout2Report
from ScoutSuite.core.exceptions import RuleExceptions
from ScoutSuite.core.ruleset import Ruleset
from ScoutSuite.core.processingengine import ProcessingEngine
from ScoutSuite.providers import get_provider


def main(args):
    """
    Main method that runs a scan

    :return:
    """

    # Get the dictionnary to get None instead of a crash
    args = args.__dict__

    # Configure the debug level
    configPrintException(args.get('debug'))

    # Create a cloud provider object
    cloud_provider = get_provider(provider=args.get('provider'),
                                  profile=args.get('profile'),
                                  project_id=args.get('project_id'),
                                  folder_id=args.get('folder_id'),
                                  organization_id=args.get('organization_id'),
                                  all_projects=args.get('all_projects'),
                                  report_dir=args.get('report_dir'),
                                  timestamp=args.get('timestamp'),
                                  services=args.get('services'),
                                  skipped_services=args.get('skipped_services'),
                                  thread_config=args.get('thread_config'),
                                  )

    report_file_name = generate_report_name(cloud_provider.provider_code, args)

    # TODO move this to after authentication, so that the report can be more specific to what's being scanned.
    # For example if scanning with a GCP service account, the SA email can only be known after authenticating...
    # Create a new report
    report = Scout2Report(args.get('provider'), report_file_name, args.get('report_dir'), args.get('timestamp'))

    # Complete run, including pulling data from provider
    if not args.get('fetch_local'):
        # Authenticate to the cloud provider
        authenticated = cloud_provider.authenticate(profile=args.get('profile'),
                                                    csv_credentials=args.get('csv_credentials'),
                                                    mfa_serial=args.get('mfa_serial'),
                                                    mfa_code=args.get('mfa_code'),
                                                    user_account=args.get('user_account'),
                                                    service_account=args.get('service_account'),
                                                    cli=args.get('cli'),
                                                    msi=args.get('msi'),
                                                    service_principal=args.get('service_principal'),
                                                    file_auth=args.get('file_auth'),
                                                    tenant_id=args.get('tenant_id'),
                                                    subscription_id=args.get('subscription_id'),
                                                    client_id=args.get('client_id'),
                                                    client_secret=args.get('client_secret'),
                                                    username=args.get('username'),
                                                    password=args.get('password')
                                                    )

        if not authenticated:
            return 42

        # Fetch data from provider APIs
        try:
            cloud_provider.fetch(regions=args.get('regions'))
        except KeyboardInterrupt:
            printInfo('\nCancelled by user')
            return 130

        # Update means we reload the whole config and overwrite part of it
        if args.get('update'):
            current_run_services = copy.deepcopy(cloud_provider.services)
            last_run_dict = report.jsrw.load_from_file(AWSCONFIG)
            cloud_provider.services = last_run_dict['services']
            for service in cloud_provider.service_list:
                cloud_provider.services[service] = current_run_services[service]

    # Partial run, using pre-pulled data
    else:
        # Reload to flatten everything into a python dictionary
        last_run_dict = report.jsrw.load_from_file(AWSCONFIG)
        for key in last_run_dict:
            setattr(cloud_provider, key, last_run_dict[key])

    # Pre processing
    cloud_provider.preprocessing(args.get('ip_ranges'), args.get('ip_ranges_name_key'))

    # Analyze config
    finding_rules = Ruleset(environment_name=args.get('profile'),
                            cloud_provider=args.get('provider'),
                            filename=args.get('ruleset'),
                            ip_ranges=args.get('ip_ranges'),
                            aws_account_id=cloud_provider.aws_account_id)
    processing_engine = ProcessingEngine(finding_rules)
    processing_engine.run(cloud_provider)

    # Create display filters
    filter_rules = Ruleset(cloud_provider=args.get('provider'),
                           filename='filters.json',
                           rule_type='filters',
                           aws_account_id=cloud_provider.aws_account_id)
    processing_engine = ProcessingEngine(filter_rules)
    processing_engine.run(cloud_provider)

    # Handle exceptions
    try:
        exceptions = RuleExceptions(args.get('profile'), args.get('exceptions')[0])
        exceptions.process(cloud_provider)
        exceptions = exceptions.exceptions
    except Exception as e:
        printDebug('Warning, failed to load exceptions. The file may not exist or may have an invalid format.')
        exceptions = {}

    # Finalize
    cloud_provider.postprocessing(report.current_time, finding_rules)

    # TODO this is AWS-specific - move to postprocessing?
    # Get organization data if it exists
    try:
        profile = AWSProfiles.get(args.get('profile'))[0]
        if 'source_profile' in profile.attributes:
            organization_info_file = os.path.join(os.path.expanduser('~/.aws/recipes/%s/organization.json' %
                                                                     profile.attributes['source_profile']))
            if os.path.isfile(organization_info_file):
                with open(organization_info_file, 'rt') as f:
                    org = {}
                    accounts = json.load(f)
                    for account in accounts:
                        account_id = account.pop('Id')
                        org[account_id] = account
                    setattr(cloud_provider, 'organization', org)
    except Exception as e:
        pass

    # Save config and create HTML report
    html_report_path = report.save(cloud_provider, exceptions, args.get('force_write'), args.get('debug'))

    # Open the report by default
    if not args.get('no_browser'):
        printInfo('Opening the HTML report...')
        url = 'file://%s' % os.path.abspath(html_report_path)
        webbrowser.open(url, new=2)

    return 0


def generate_report_name(provider_code, args):
    if provider_code == 'aws':
        if args.get('profile'):
            report_file_name = 'aws-%s' % args.get('profile')[0]
        else:
            report_file_name = 'aws'
    if provider_code == 'gcp':
        if args.get('project_id'):
            report_file_name = 'gcp-%s' % args.get('project_id')
        elif args.get('organization_id'):
            report_file_name = 'gcp-%s' % args.get('organization_id')
        elif args.get('folder_id'):
            report_file_name = 'gcp-%s' % args.get('folder_id')
        else:
            report_file_name = 'gcp'
    if provider_code == 'azure':
        report_file_name = 'azure'
    return report_file_name



# -*- coding: utf-8 -*-

import copy

from opinel.utils.console import printError, printException


########################################
# Functions
########################################

def combine_paths(path1, path2):
    path = path1
    for p in path2:
        if p == '..':
            del (path[-1])
        else:
            path.append(p)
    return path


def get_attribute_at(config, target_path, key, default_value=None):
    """
    Return attribute value at a given path.

    :param config:
    :param target_path:
    :param key:
    :param default_value:
    :return:
    """
    for target in target_path:
        config = config[target]
    return config[key] if key in config else default_value


def get_object_at(object, path, attribute_name=None):
    """
    Get arbitrary object given a dictionary and path (list of keys).

    :param dictionary:
    :param path:
    :param attribute_name:
    :return:
    """
    o = object
    try:
        for p in path:
            if type(o) is dict:
                o = o[p]
            else:
                o = getattr(o, p)

        if attribute_name:
            if type(o) is dict:
                return o[attribute_name]
            else:
                return getattr(o, attribute_name)
        else:
            return o
    except Exception as e:
        raise Exception


def get_value_at(all_info, current_path, key, to_string=False):
    """
    Get value located at a given path.

    :param all_info:        All of the services' data
    :param current_path:    The value of the `path` variable defined in the finding file
    :param key:             The key that is being requested
    :param to_string:       Whether or not the returned value should be casted as a string
    :return:                The value in `all_info` indicated by the `key` in `current_path`
    """
    keys = key.split('.')
    if keys[-1] == 'id':
        target_obj = current_path[len(keys) - 1]
    else:
        if key == 'this':
            target_path = current_path
        elif '.' in key:
            target_path = []
            for i, key in enumerate(keys):
                # If 'id', replace by value
                if key == 'id':
                    target_path.append(current_path[i])
                # If empty key and value is an index, keep the index
                elif key == '' and i < len(current_path) and current_path[i].isdigit():
                    target_path.append(int(current_path[i]))
                # Otherwise, use key
                else:
                    target_path.append(key)
            if len(keys) > len(current_path):
                target_path = target_path + keys[len(target_path):]
        else:
            target_path = copy.deepcopy(current_path)
            target_path.append(key)
        target_obj = all_info
        for p in target_path:
            try:
                if type(target_obj) == list and type(target_obj[0]) == dict:
                    target_obj = target_obj[int(p)]
                # TODO ensure this additional condition didn't break anything
                elif type(target_obj) == list and type(p) == int:
                    target_obj = target_obj[p]
                elif type(target_obj) == list:
                    target_obj = p
                elif p == '':
                    target_obj = target_obj
                else:
                    try:
                        target_obj = target_obj[p]
                    except Exception as e:
                        printError('Current path: %s' % str(current_path))
                        printException(e)
                        raise Exception
            except Exception as e:
                printError('Current path: %s' % str(current_path))
                printException(e)
                raise Exception
    if to_string:
        return str(target_obj)
    else:
        return target_obj
